<?php

namespace Netzmacher\Caddy\Lib\Caddy;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2013-2020 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  Caddy is a fork of wt_cart (version 1.4.6)
 *  (c) wt_cart 2010-2012 - wt_cart Development Team <info@wt-cart.com>
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Plugin for the 'caddy' extension.
 *
 * @author	Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage    tx_caddy
 * @version     7.5.8
 * @since       1.4.6
 */
class Minmax
{

	public $extKey = 'caddy';
	public $prefixId = 'tx_caddy_pi1';
	public $scriptRelPath = 'Resources/Private/Language/Plugins/pi1/class.tx_caddy_pi1.php';
	private $pObj = null; // Object : the parent object
	private $conf = null; // Array  : the TypoScript configuration of the parent object
	public $cObj = null; // Object : the content object of the parent object
	private $drs = null; // Object : the DRS object of the parent object

	/**
	 * _evaluateCaddyMax( )  : Checks max limit depending on the caddy (plugin/flexform)
	 *                                         while items are updating
	 *                                         If the limit is passed over, quantity will decreased and
	 *                                         an error prompt will be near the item.
	 *                                         It's possible, that the quantity of more than one item
	 *                                         will decreased.
	 *
	 * @param	array		$product  : the current product
	 * @return	array		$product  : the current or the updated product
	 * @access private
	 * @version 7.5.8
	 * @since 2.0.0
	 */

	private function _evaluateCaddyMax( $product )
	{
		$itemsQuantityMax = $this->pObj->pObj->flexform->originMax;

		$productUnlimited = $this->_evaluateCaddyMaxUnlimited( $product, $itemsQuantityMax );
		if( $productUnlimited )
		{
			return $productUnlimited;
		}

		$productProper = $this->_evaluateCaddyMaxProper( $product, $itemsQuantityMax );
		if( $productProper )
		{
			return $productProper;
		}

		$productLimited = $this->_evaluateCaddyMaxLimited( $product, $itemsQuantityMax );
		return $productLimited;
	}

	/**
	 * _evaluateCaddyMaxLimited( )  : 
	 *
	 * @param	array		$product  : the current product
	 * @return	array		$product  : the current or the updated product
	 * @access private
	 * @version 7.5.8
	 * @since 2.0.0
	 */
	private function _evaluateCaddyMaxLimited( $product, $itemsQuantityMax )
	{
		$itemsQuantity = $this->_quantityGet();
		$itemsQuantityOverrun = $itemsQuantity - $itemsQuantityMax;

		$quantity = $product[ 'qty' ] - $itemsQuantityOverrun;
		if( $quantity < 1 )
		{
			$quantity = 1;
		}
		$product[ 'qty' ] = $this->_productSetQuantity( $quantity, $product[ 'uid' ] );

		$llKey = 'error_itemsMax';
		$llAlt = 'No value for error_itemsMax in ' . __METHOD__ . ' (' . __LINE__ . ')';
		$llPrompt = $this->pObj->pObj->pi_getLL( $llKey, $llAlt );
		$llPrompt = sprintf( $llPrompt, $itemsQuantityMax, $itemsQuantityMax );
		$product[ 'error' ][ 'itemsMax' ] = $llPrompt;

		if( !$this->drs->drsQuantity )
		{
			return $product;
		}

		$prompt = 'Maximum limit for all items is overrun: Items #' . $itemsQuantity . ', limit #' . $itemsQuantityMax;
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );
		$prompt = 'Quantity for item  (' . $product[ 'title' ] . ': ' . $product[ 'uid' ] . ') '
						. 'will setup to #' . $product[ 'qty' ];
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );

		return $product;
	}

	/**
	 * _evaluateCaddyMaxProper( )  : 
	 *
	 * @param	array		$product  : the current product
	 * @return	array		$product  : the current or the updated product
	 * @access private
	 * @version 7.5.8
	 * @since 2.0.0
	 */
	private function _evaluateCaddyMaxProper( $product, $itemsQuantityMax )
	{
		$itemsQuantity = $this->_quantityGet();

		if( $itemsQuantity > $itemsQuantityMax )
		{
			return;
		}

		unset( $product[ 'error' ][ 'itemsMax' ] );

		if( !$this->drs->drsQuantity )
		{
			return $product;
		}

		$prompt = 'Maximum limit for all items isn\'t overrun: Items #' . $itemsQuantity . ', limit #' . $itemsQuantityMax;
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );

		return $product;
	}

	/**
	 * _evaluateCaddyMaxUnlimited( )  : 
	 *
	 * @param	array		$product  : the current product
	 * @return	array		$product  : the current or the updated product
	 * @access private
	 * @version 7.5.8
	 * @since 2.0.0
	 */
	private function _evaluateCaddyMaxUnlimited( $product, $itemsQuantityMax )
	{
		if( !empty( $itemsQuantityMax ) )
		{
			return;
		}

		unset( $product[ 'error' ][ 'itemsMax' ] );

		if( !$this->drs->drsQuantity )
		{
			return $product;
		}

		$prompt = 'No maximum limit is given in the plugin/flexform. Maximum limit for all items won\'t checked.';
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );

		return $product;
	}

	/**
	 * _evaluateCaddyMin( )  : Checks min limit depending on the caddy (plugin/flexform)
	 *                                         while items are updating
	 *                                         If the limit is undercut, quantity will increased and
	 *                                         an error prompt will be near the item.
	 *                                         It's possible, that the quantity of more than one item
	 *                                         will increased.
	 *
	 * @param	array		$product  : the current product
	 * @return	array		$product  : the current or the updated product
	 * @access private
	 * @version 7.5.8
	 * @since 2.0.0
	 */
	private function _evaluateCaddyMin( $product )
	{
		$itemsQuantityMin = $this->pObj->pObj->flexform->originMin;


		$productUnlimited = $this->_evaluateCaddyMinUnlimited( $product, $itemsQuantityMin );
		if( $productUnlimited )
		{
			return $productUnlimited;
		}

//		$productWoLimit = $this->_evaluateCaddyMinWoLimit( $product, $itemsQuantityMin );
//		if( $productWoLimit )
//		{
//			return $productWoLimit;
//		}
//
		$itemsQuantity = $this->_quantityGet();

		$productProper = $this->_evaluateCaddyMinProper( $product, $itemsQuantityMin, $itemsQuantity );
		if( $productProper )
		{
			return $productProper;
		}

		$productLimited = $this->_evaluateCaddyMinLimited( $product, $itemsQuantityMin, $itemsQuantity );
		return $productLimited;
	}

	/**
	 * _evaluateCaddyMinProper( )  : 
	 *
	 * @param	array		$product  : the current product
	 * @return	array		$product  : the current or the updated product
	 * @access private
	 * @version 7.5.8
	 * @since 2.0.0
	 */
	private function _evaluateCaddyMinProper( $product, $itemsQuantityMin, $itemsQuantity )
	{

		// RETURN : limit for min _quantity for all items isn't passed
		if( $itemsQuantity < $itemsQuantityMin )
		{
			return;
		}

		unset( $product[ 'error' ][ 'itemsMin' ] );

		if( !$this->drs->drsQuantity )
		{
			return $product;
		}

		$prompt = 'Minimum limit for all items isn\'t undercut: Items #' . $itemsQuantity . ', limit #' . $itemsQuantityMin;
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );
		return $product;
	}

	/**
	 * _evaluateCaddyMinLimited( )  : 
	 *
	 * @param	array		$product  : the current product
	 * @return	array		$product  : the current or the updated product
	 * @access private
	 * @version 7.5.8
	 * @since 2.0.0
	 */
	private function _evaluateCaddyMinLimited( $product, $itemsQuantityMin, $itemsQuantity )
	{
		$itemsQuantityUndercut = $itemsQuantityMin - $itemsQuantity;

		$product[ 'qty' ] = $this->_productGetDefaultQty( $product[ 'qty' ] );
		$quantity = $product[ 'qty' ] + $itemsQuantityUndercut;
		$product[ 'qty' ] = $this->_productSetQuantity( $quantity, $product[ 'uid' ] );

		$llKey = 'error_itemsMin';
		$llAlt = 'No value for error_itemsMin in ' . __METHOD__ . ' (' . __LINE__ . ')';
		$llPrompt = $this->pObj->pObj->pi_getLL( $llKey, $llAlt );
		$llPrompt = sprintf( $llPrompt, $itemsQuantityMin, $itemsQuantityMin );
		$product[ 'error' ][ 'itemsMin' ] = $llPrompt;

		if( !$this->drs->drsQuantity )
		{
			return $product;
		}

		$prompt = 'Minimum limit for all items is undercut: Items #' . $itemsQuantity . ', limit #' . $itemsQuantityMin;
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );
		$prompt = 'Quantity for item  (' . $product[ 'title' ] . ': ' . $product[ 'uid' ] . ') '
						. 'will setup to #' . $product[ 'qty' ];
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );

		return $product;
	}

	/**
	 * _evaluateCaddyMinWoLimit( )  : 
	 *
	 * @param	array		$product  : the current product
	 * @return	array		$product  : the current or the updated product
	 * @access private
	 * @version 7.5.8
	 * @since 2.0.0
	 */
	private function _evaluateCaddyMinWoLimit( $product, $itemsQuantityMin )
	{
		if( !empty( $itemsQuantityMin ) )
		{
			return;
		}

		unset( $product[ 'error' ][ 'itemsMin' ] );

		if( !$this->drs->drsQuantity )
		{
			return $product;
		}

		$prompt = 'No minimum limit is given in the plugin/flexform. Minimum limit for all items won\'t checked.';
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );
		return $product;
	}

	/**
	 * _evaluateCaddyMinUnlimited( )  : 
	 *
	 * @param	array		$product  : the current product
	 * @return	array		$product  : the current or the updated product
	 * @access private
	 * @version 7.5.8
	 * @since 2.0.0
	 */
	private function _evaluateCaddyMinUnlimited( $product, $itemsQuantityMin )
	{
		if( !empty( $itemsQuantityMin ) )
		{
			return;
		}

		unset( $product[ 'error' ][ 'itemsMin' ] );

		if( !$this->drs->drsQuantity )
		{
			return $product;
		}

		$prompt = 'No minimum limit is given in the plugin/flexform. Minimum limit for all items won\'t checked.';
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );

		return $product;
	}

	/**
	 * _evaluateDrs( ) : Prompt to the DRS the current case.
	 *                             Possible case is
	 *                             * add
	 *                             * delete
	 *                             * update
	 *
	 * @return	void
	 * @access private
	 * @version 6.3.0
	 * @since 2.0.0
	 */
	private function _evaluateDrs()
	{
		// SWITCH : add, update, delete
		switch( true )
		{
			case( $this->pObj->pObj->gpvar[ 'uid' ] ):
				$prompt = 'Case: add an item';
				// #i0100, 151219, dwildt, 1-
				//\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );
				break;
			case( $this->pObj->pObj->piVars[ 'qty' ] ):
				$prompt = 'Case: update item quantity';
				// #i0100, 151219, dwildt, 1-
				//\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );
				break;
			case( $this->pObj->pObj->piVars[ 'del' ] ):
				$prompt = 'Case: delete an item';
				// #i0100, 151219, dwildt, 1-
				//\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );
				break;
			default:
				$prompt = 'ERROR: no value for switch' . PHP_EOL .
								'Sorry for the trouble.<br />' . PHP_EOL .
								'TYPO3 Caddy<br />' . PHP_EOL .
								__METHOD__ . ' (' . __LINE__ . ')';
				die( $prompt );
				break;
		}
		// SWITCH : add, update, delete
		// #i0100, 151219, dwildt, 5+
		if( !$this->drs->drsQuantity )
		{
			return;
		}
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );

		return;
	}

	/**
	 * _evaluateItemMax( ) : Checks, if the maximum quantity is within the limit.
	 *                                 If not, quantity will decreased to the limit,
	 *                                 and the item will get an error prompt
	 *
	 * @param	array		$product  : the current product
	 * @return	array		$product  : the current or the updated product
	 * @access private
	 * @version 7.5.8
	 * @since 2.0.0
	 */
	private function _evaluateItemMax( $product )
	{
		$productUnlimited = $this->_evaluateItemMaxUnlimited( $product );
		if( $productUnlimited )
		{
			return $productUnlimited;
		}

		// SWITCH : limit is overrun or limit isn't overrun
		switch( true )
		{
			case( $product[ 'qty' ] > $product[ 'max' ] ):
				$product = $this->_evaluateItemMaxLimited( $product );
				break;
			case( $product[ 'qty' ] <= $product[ 'max' ] ):
			default:
				$product = $this->_evaluateItemMaxProper( $product );
				break;
		}
		// SWITCH : limit is overrun or limit isn't overrun

		return $product;
	}

	/**
	 * _evaluateItemMaxLimited( ) : 
	 *
	 * @param	array		$product  : the current product
	 * @return	array		$product  : the current or the updated product
	 * @access private
	 * @version 7.5.8
	 * @since 2.0.0
	 */
	private function _evaluateItemMaxLimited( $product )
	{
		$product[ 'qty' ] = $this->_productSetQuantity( $product[ 'max' ], $product[ 'uid' ] );
		$llKey = 'error_max';
		$llAlt = 'No value for error_max in ' . __METHOD__ . ' (' . __LINE__ . ')';
		$llPrompt = $this->pObj->pObj->pi_getLL( $llKey, $llAlt );
		$llPrompt = sprintf( $llPrompt, $product[ 'max' ], $product[ 'max' ] );
		$product[ 'error' ][ 'max' ] = $llPrompt;
		$this->pObj->pObj->piVars[ 'qty' ][ 'uid=' . $product[ 'uid' ] ] = $product[ 'qty' ];

		if( $this->drs->drsQuantity )
		{
			return $product;
		}

		$prompt = 'Maximum limit of the current item (' . $product[ 'title' ] . ': ' . $product[ 'uid' ] . ') is overrun. Item #' . $product[ 'qty' ] . ', limit #' . $product[ 'max' ];
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );
		$prompt = 'Quantity will setup to #' . $product[ 'max' ];
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );

		return $product;
	}

	/**
	 * _evaluateItemMaxProper( ) : 
	 *
	 * @param	array		$product  : the current product
	 * @return	array		$product  : the current or the updated product
	 * @access private
	 * @version 7.5.8
	 * @since 2.0.0
	 */
	private function _evaluateItemMaxProper( $product )
	{
		unset( $product[ 'error' ][ 'max' ] );

		if( $this->drs->drsQuantity )
		{
			return $product;
		}

		$prompt = 'Maximum limit of the current item (' . $product[ 'title' ] . ': ' . $product[ 'uid' ] . ') isn\'t overrun. Item #' . $product[ 'qty' ] . ', limit #' . $product[ 'max' ];
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );

		return $product;
	}

	/**
	 * _evaluateItemMaxUnlimited( ) : 
	 *
	 * @param	array		$product  : the current product
	 * @return	array		$product  : the current or the updated product
	 * @access private
	 * @version 7.5.8
	 * @since 2.0.0
	 */
	private function _evaluateItemMaxUnlimited( $product )
	{
		// RETURN : current item hasn't any max quantity limit
		if( !empty( $product[ 'max' ] ) )
		{
			return;
		}

		unset( $product[ 'error' ][ 'max' ] );

		if( !$this->drs->drsQuantity )
		{
			return $product;
		}

		// DRS
		$prompt = 'Current item (' . $product[ 'title' ] . ': ' . $product[ 'uid' ] . ') hasn\'t any maximum limit. Maximum limit won\'t checked.';
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );

		return $product;
	}

	/**
	 * _evaluateItemMin( ) : Checks, if the minimum quantity is within the limit.
	 *                                 If not, quantity will increased to the limit,
	 *                                 and the item will get an error prompt
	 *
	 * @param	array		$product  : the current product
	 * @return	array		$product  : the current or the updated product
	 * @access private
	 * @version 7.5.8
	 * @since 2.0.0
	 */
	private function _evaluateItemMin( $product )
	{
		$productUnlimited = $this->_evaluateItemMinUnlimited( $product );
		if( $productUnlimited )
		{
			return $productUnlimited;
		}

		switch( true )
		{
			case( $product[ 'qty' ] < $product[ 'min' ] ):
				$product = $this->_evaluateItemMinLimited( $product );
				break;
			case( $product[ 'qty' ] >= $product[ 'min' ] ):
			default:
				$product = $this->_evaluateItemMinProper( $product );
				break;
		}

		return $product;
	}

	/**
	 * _evaluateItemMinLimited( ) : 
	 *
	 * @param	array		$product  : the current product
	 * @return	array		$product  : the current or the updated product
	 * @access private
	 * @version 7.5.8
	 * @since 2.0.0
	 */
	private function _evaluateItemMinLimited( $product )
	{
		$product[ 'qty' ] = $this->_productSetQuantity( $product[ 'min' ], $product[ 'uid' ] );
		$llKey = 'error_min';
		$llAlt = 'No value for error_min in ' . __METHOD__ . ' (' . __LINE__ . ')';
		$llPrompt = $this->pObj->pObj->pi_getLL( $llKey, $llAlt );
		$llPrompt = sprintf( $llPrompt, $product[ 'min' ], $product[ 'min' ] );
		$product[ 'error' ][ 'min' ] = $llPrompt;
		$this->pObj->pObj->piVars[ 'qty' ][ 'uid=' . $product[ 'uid' ] ] = 0;

		if( !$this->drs->drsQuantity )
		{
			return $product;
		}

		$prompt = 'Minimum limit of the current item (' . $product[ 'title' ] . ': '
						. $product[ 'uid' ] . ') is undercut. Item #' . $product[ 'qty' ] . ', limit #' . $product[ 'min' ];
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );
		$prompt = 'Quantity will setup to #' . $product[ 'min' ];
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );

		return $product;
	}

	/**
	 * _evaluateItemMinProper( ) : 
	 *
	 * @param	array		$product  : the current product
	 * @return	array		$product  : the current or the updated product
	 * @access private
	 * @version 7.5.8
	 * @since 2.0.0
	 */
	private function _evaluateItemMinProper( $product )
	{
		unset( $product[ 'error' ][ 'min' ] );

		if( !$this->drs->drsQuantity )
		{
			return $product;
		}

		$prompt = 'Minimum limit of the current item (' . $product[ 'title' ] . ': ' . $product[ 'uid' ] . ') '
						. 'isn\'t undercut. Item #' . $product[ 'qty' ] . ', limit #' . $product[ 'min' ];
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );

		return $product;
	}

	/**
	 * _evaluateItemMinUnlimited( ) : 
	 *
	 * @param	array		$product  : the current product
	 * @return	array		$product  : the current or the updated product
	 * @access private
	 * @version 7.5.8
	 * @since 2.0.0
	 */
	private function _evaluateItemMinUnlimited( $product )
	{
// RETURN : current item hasn't any min _quantity limit
		if( !empty( $product[ 'min' ] ) )
		{
			return;
		}

		unset( $product[ 'error' ][ 'min' ] );

		if( !$this->drs->drsQuantity )
		{
			return $product;
		}

		$prompt = 'Current item (' . $product[ 'title' ] . ': ' . $product[ 'uid' ] . ') hasn\'t any minimum limit. Minimum limit won\'t checked.';
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );

		return $product;
	}

	/**
	 * _evaluateStock( )  :
	 *
	 * @param	array		$product  : the current product
	 * @return	array		$product  : the current or the updated product
	 * @access private
	 * @version 4.0.7
	 * @since 4.0.7
	 */
	private function _evaluateStock( $product )
	{
		$qtyByOrder = $product[ 'qty' ];
		$qtyInStock = $this->stockmanager->getItemQuantity( $product );
//var_dump(__METHOD__, __LINE__, $qtyInStock );

		if( $qtyByOrder <= $qtyInStock )
		{
			unset( $product[ 'error' ][ 'itemStock' ] );
			return $product;
		}

		$product[ 'qty' ] = $qtyInStock;

// DRS
		if( $this->drs->drsQuantity )
		{
			$prompt = 'Quantity by order #' . $qtyByOrder . ' is greater than _quantity in stock #' . $qtyInStock . '. '
							. 'Quantity by order is set to  #' . $qtyInStock . '.'
							. 'Item: ' . $product[ 'title' ] . ': ' . $product[ 'uid' ] . '.'
			;
			\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );
		}
// DRS
// Set the error prompt
		$llKey = 'error_itemStockMax';
		if( $qtyInStock == 1 )
		{
			$llKey = 'error_itemStockMaxOne';
		}
		$llAlt = 'No value for error_itemStock in ' . __METHOD__ . ' (' . __LINE__ . ')';
		$llPrompt = $this->pObj->pObj->pi_getLL( $llKey, $llAlt );
		$llPrompt = sprintf( $llPrompt, $qtyInStock, $qtyInStock );
		$product[ 'error' ][ 'itemStock' ] = $llPrompt;
// Set the error prompt

		return $product;
	}

	/**
	 * _init( )
	 *
	 * @return	void
	 * @access private
	 * @internal   #55726
	 * @version    4.0.7
	 * @since      4.0.7
	 */
	private function _init()
	{
		$this->_initVars();
		$this->_initInstances();
	}

	/**
	 * _initInstances( )
	 *
	 * @return	void
	 * @access private
	 * @internal   #55726
	 * @version    4.0.7
	 * @since      4.0.7
	 */
	private function _initInstances()
	{
		if( !( $this->_initInstances === null ) )
		{
			return;
		}

		$this->_initInstancesStockmanager();

		$this->_initInstances = true;
	}

	/**
	 * _initInstancesStockmanager( )
	 *
	 * @return	void
	 * @access private
	 * @internal   #55726
	 * @version    4.0.7
	 * @since      4.0.7
	 */
	private function _initInstancesStockmanager()
	{
		// #i0226, 180312, dwildt, ~
		$this->stockmanager = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance( \Netzmacher\Caddy\Lib\Caddy\Stockmanager::class );
		$this->stockmanager->setParentObject( $this );
	}

	/**
	 * _initVars( )
	 *
	 * @return	void
	 * @access private
	 * @internal   #55726
	 * @version    4.0.7
	 * @since      4.0.7
	 */
	private function _initVars()
	{
		$this->conf = $this->pObj->pObj->conf;
		$this->cObj = $this->pObj->pObj->cObj;
	}

	/**
	 * _productGetDefaultQty( )  : Returns 1 if the quantity of the added product is 0
	 *
	 * @param	integer		$qty: quantity of the added product
	 * @return	integer		$qty: 1 if the quantity of the added product is 0
	 * @access private
	 * @internal #i0079
	 * @version 6.0.9
	 * @since 6.0.9
	 */
	private function _productGetDefaultQty( $qty )
	{
		if( !empty( $qty ) )
		{
			return $qty;
		}

		if( $this->pObj->piVars[ 'del' ] )
		{
			return 0;
		}

		return 1;
	}

	/**
	 * _productSetQuantity( )  : Returns the given _quantity
	 *
	 * @param	integer		$quantity : current quantity of the current product
	 * @param	integer		$uid      : uid of the current product
	 * @return	integer		$quantity : current quantity of the current product
	 * @access private
	 * @version     7.5.8
	 * @since       2.0.0
	 */
	private function _productSetQuantity( $quantity, $uid )
	{
		//var_dump(__METHOD__, __LINE__, $this->pObj->pObj->piVars, $this->pObj->pObj->gpvar);
		switch( true )
		{
			case( $this->pObj->pObj->gpvar[ 'uid' ] ):
				// add an item
				$this->pObj->pObj->gpvar[ 'qty' ] = $quantity;
				$this->_productSetQuantityDRSAdd( $quantity );
				break;
			case( isset($this->pObj->pObj->piVars[ 'qty' ][ 'uid=' . $uid ]) ):
				$this->pObj->pObj->piVars[ 'qty' ][ 'uid=' . $uid ] = $quantity;
				$this->_productSetQuantityDRSUpdate( $quantity );
				break;
			case( $this->pObj->pObj->piVars[ 'qty' ] ):
				$this->pObj->pObj->piVars[ 'qty' ][ $uid ] = $quantity;
				$this->_productSetQuantityDRSUpdate( $quantity );
				break;
			case( $this->pObj->pObj->piVars[ 'del' ] ):
				$this->_productSetQuantityDRSDelete( );
			default:
				$prompt = 'ERROR: no value for switch' . PHP_EOL .
								'Sorry for the trouble.<br />' . PHP_EOL .
								'TYPO3 Caddy<br />' . PHP_EOL .
								__METHOD__ . ' (' . __LINE__ . ')';
				die( $prompt );
				break;
		}

		return $quantity;
	}

	/**
	 * _productSetQuantityDRSAdd( )
	 *
	 * @param	integer		$quantity : current quantity of the current product
	 * @return	void
	 * @access private
	 * @version     7.5.8
	 * @since       2.0.0
	 */
	private function _productSetQuantityDRSAdd( $quantity )
	{
		if( !$this->drs->drsQuantity )
		{
			return;
		}
		$prompt = 'Case: add an item';
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );
//var_dump(__METHOD__, __LINE__, $prompt );		
		$prompt = 'Quantity of current product is set to ' . $quantity;
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );
//var_dump(__METHOD__, __LINE__, $prompt );		
		return;
	}

	/**
	 * _productSetQuantityDRSDelete( )  : 
	 *
	 * @return	void
	 * @access private
	 * @version     7.5.8
	 * @since       2.0.0
	 */
	private function _productSetQuantityDRSDelete( )
	{
		if( !$this->drs->drsQuantity )
		{
			return;
		}
		$prompt = 'Case: update quantity after delete';
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );
	}

	/**
	 * _productSetQuantityDRSUpdate( )
	 *
	 * @param	integer		$quantity : current quantity of the current product
	 * @return	void
	 * @access private
	 * @version     7.5.8
	 * @since       2.0.0
	 */
	private function _productSetQuantityDRSUpdate( $quantity )
	{
		if( !$this->drs->drsQuantity )
		{
			return;
		}
		$prompt = 'Case: update quantity';
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );
//var_dump(__METHOD__, __LINE__, $prompt );		
		$prompt = 'Quantity of current product is set to ' . $quantity;
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );
//var_dump(__METHOD__, __LINE__, $prompt );
	}

	/**
	 * _quantityGet( )  :
	 *
	 * @return	integer		$quantity : the quantity of the current items
	 * @access private
	 * @version 2.0.0
	 * @since 2.0.0
	 */
	private function _quantityGet()
	{
		$quantity = 0;

// SWITCH : add an item or update items quantity
		switch( true )
		{
			case( $this->pObj->pObj->gpvar[ 'uid' ] ):
				$quantity = $this->_quantityGetAdd();
				break;
			case( $this->pObj->pObj->piVars[ 'qty' ] ):
				$quantity = $this->_quantityGetUpdate();
				break;
			case( $this->pObj->pObj->piVars[ 'del' ] ):
				$quantity = $this->_quantityGetDelete();
				break;
			default:
				$prompt = 'ERROR: no value for switch.<br />' . PHP_EOL .
								'Sorry for the trouble.<br />' . PHP_EOL .
								'TYPO3 Caddy<br />' . PHP_EOL .
								__METHOD__ . ' (' . __LINE__ . ')';
				die( $prompt );
		}

		return $quantity;
	}

	/**
	 * _quantityGetAdd( )  :
	 *
	 * @return	integer		$quantity : the quantity of the current items
	 * @access private
	 * @version 2.0.0
	 * @since 2.0.0
	 */
	private function _quantityGetAdd()
	{
// Default value
		$quantityAdd = ( int ) $this->pObj->pObj->gpvar[ 'qty' ];
		$quantityAll = 0;
		$quantitySum = 0;

// DRS
		if( $this->drs->drsQuantity )
		{
			$prompt = 'Quantity of current product should increased by: ' . $quantityAdd;
			\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );
		}
// DRS
// Get products
		$products = $this->pObj->productsGet();

// SWITCH : products or any product
		switch( true )
		{
			case(!empty( $products ) ):
				foreach( ( array ) $products as $product )
				{
					$quantityAll = $quantityAll + $product[ 'qty' ]
					;
				}
				break;
			case( empty( $products ) ):
			default:
				//$quantity = ( int ) $this->pObj->pObj->gpvar['qty'];
				break;
		}
// SWITCH : products or any product

		$quantitySum = $quantityAll + $quantityAdd;
// DRS
		if( $this->drs->drsQuantity )
		{
			$prompt = 'Quantity: all #' . $quantityAll . ' + add #' . $quantityAdd . ' = sum #' . $quantitySum;
			\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/QUANTITY] ' . $prompt, $this->extKey, 0 );
		}
// DRS
		return $quantitySum;
	}

	/**
	 * _quantityGetDelete( )  :
	 *
	 * @return	integer		$quantity : the quantity of the current items
	 * @access private
	 * @version 2.0.0
	 * @since 2.0.0
	 */
	private function _quantityGetDelete()
	{
		$quantity = 0;

		$products = $this->productsGet();

		foreach( ( array ) $products as $product )
		{
			$quantity = $quantity + $product[ 'qty' ]
			;
		}

		return $quantity;
	}

	/**
	 * _quantityGetUpdate( )  :
	 *
	 * @return	integer		$quantity : the quantity of the current items
	 * @access private
	 * @version 2.0.0
	 * @since 2.0.0
	 */
	private function _quantityGetUpdate()
	{
		$quantity = 0;

		foreach( ( array ) $this->pObj->pObj->piVars[ 'qty' ] as $value )
		{
//var_dump( __METHOD__, __LINE__, $value );
			$quantity = $quantity + ( int ) $value
			;
		}

		return $quantity;
	}

	/**
	 * _setParentObjectDrs( )  :
	 *
	 * @return	void
	 * @access private
	 * @version    7.5.8
	 * @since      7.5.8
	 */
	private function _setParentObjectDrs()
	{
		if( is_object( $this->pObj->pObj->drs ) )
		{
			$this->drs = $this->pObj->pObj->drs;
			return;
		}

		$prompt = 'ERROR: no DRS object!<br />' . PHP_EOL .
						'Sorry for the trouble.<br />' . PHP_EOL .
						'TYPO3 Caddy<br />' . PHP_EOL .
						__METHOD__ . ' (' . __LINE__ . ')';
		die( $prompt );
	}

	/**
	 * evaluate( )  : Checks
	 *                           * min and max limits depending on an item (database)
	 *                           * min and max limits depending on the caddy (plugin/flexform)
	 *                           If a limit is passed over, quantities will updated and there will be
	 *                           error prompts near the items.
	 *                           min and max limits of the caddy have precedence!
	 *                           Example:
	 *                           * An item have a maximum limit of 2
	 *                           * The caddy have a minimum limit of 4
	 *                           * _evaluate( ) will update the quantity to 4 of the product
	 *
	 * @param	array		$product  : the current product
	 * @return	array		$product  : the current or the updated product
	 * @access public
	 * @version 7.5.8
	 * @since 2.0.0
	 */
	public function evaluate( $product )
	{
		$this->_init();
		//var_dump( __METHOD__, __LINE__, $this->pObj->pObj->piVars[ 'qty' ], $product);
		switch( TRUE )
		{
			case( is_array( $this->pObj->pObj->conf[ 'settings.' ][ 'variant.' ] ) ):
				// #61877, 140916, dwildt, 5+
				// RETURN without any check: variants are configured
				return $product;
			// #i0080, 150212, dwildt, +
			case(empty( $product )):
				return NULL;
// #i0174, 171125, dwildt, 9-
//			// #i0080, 150212, dwildt, +
//			case(isset( $this->pObj->pObj->piVars[ 'qty' ] )):
//				if( empty( $this->pObj->pObj->piVars[ 'qty' ][ $product[ 'uid' ] ] ) )
//				{
//		var_dump(__METHOD__, __LINE__, $product);
//					return NULL;
//				}
//		var_dump(__METHOD__, __LINE__, $product);
//				break;
			default:
				// follow the workflow
				break;
		}

		// Prompt case add, update or delete to DRS
		$this->_evaluateDrs();

		unset( $product[ 'error' ][ 'itemsMin' ] );
		unset( $product[ 'error' ][ 'itemsMax' ] );

		// Checks the min and max limit depending on an item (database)
		$product = $this->_evaluateItemMax( $product );
		$product = $this->_evaluateItemMin( $product );

// #i0179, 171127, dwildt, 3-		
//		// Checks the min and max limit depending on the caddy (plugin/flexform)
//		$product = $this->_evaluateCaddyMax( $product );
//		$product = $this->_evaluateCaddyMin( $product );

		// #55726, 140206, dwildt, 2+
		// Checks the requirments of the stock management
		$product = $this->_evaluateStock( $product );
		return $product;
	}

	/**
	 * getConf( ) : Get the TypoScript configuration array
	 *
	 * @return	array   $conf : TypoScript configuration
	 * @access public
	 * @internal   #55726
	 * @version    4.0.7
	 * @since      4.0.7
	 */
	public function getConf()
	{
		return $this->conf;
	}

	/**
	 * getDrs( ) : Get the DRS object
	 *
	 * @return	object   $drs : DRS object
	 * @access public
	 * @internal   #55726
	 * @version    4.0.7
	 * @since      4.0.7
	 */
	public function getDrs()
	{
		return $this->drs;
	}

	/**
	 * setParentObject( )  : Returns a caddy with HTML form and HTML options among others
	 *
	 * @param	[type]		$$pObj: ...
	 * @return	void
	 * @access public
	 * @version    7.5.8
	 * @since      7.5.8
	 */
	public function setParentObject( $pObj )
	{
		if( !is_object( $pObj ) )
		{
			$prompt = 'ERROR: no parent object!<br />' . PHP_EOL .
							'Sorry for the trouble.<br />' . PHP_EOL .
							'TYPO3 Caddy<br />' . PHP_EOL .
							__METHOD__ . ' (' . __LINE__ . ')';
			die( $prompt );
		}
		$this->pObj = $pObj;

		$this->_setParentObjectDrs();
	}

}