<?php

namespace Netzmacher\Caddy\Lib\EPayment\Paymill;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2018-2020 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Class for autloading the Paymill PHP API from the extension Paymill (paymill).
 * Class defines PAYMILL_API_KEY.
 * 
 * @author			Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package			TYPO3
 * @subpackage  tx_caddy
 * @internal		#t2145
 * @version     8.10.3
 * @since       8.10.3
 */
class Autoload
{

	/**
	 * @var object
	 */
	private $_pObj;

	/**
	 * @param			object	$pObj		Parent object
	 * @return		void
	 * @access		public
	 * @version   8.10.3
	 * @since     8.10.3
	 */
	public function __construct( $pObj )
	{
		$this->_pObj = $pObj;
		$this->_dieRequirements();
	}

	/**
	 * Method for autloading the Paymill PHP API from the extension Paymill (paymill).
	 * Method defines PAYMILL_API_KEY.
	 * 
	 * @return		void
	 * @access		public
	 * @version   8.10.3
	 * @since     8.10.3
	 */
	public function PaymillPhpAPI()
	{
		// RETURN: Paymill PHP Api is loaded
		if( method_exists( '\Paymill\Request', 'create' ) )
		{
			return;
		}

		// Autload the Paymill PHP API
		$phpAutoload = $this->_Path();
		require_once( $phpAutoload );

		// Define PAYMILL_API_KEY.
		$this->_DefinePaymillApiKey();
		$this->_DefinePaymillPublicKey();
	}

	/**
	 * @return			void
	 * @access			private
	 * @version     8.10.3
	 * @since       8.10.3
	 */
	private function _DefinePaymillApiKey()
	{
		// conf respects test and live mode
		$name = $this->_pObj->conf[ 'api.' ][ 'e-payment.' ][ 'provider.' ][ 'paymill.' ][ 'keys.' ][ 'private' ];
		$conf = $this->_pObj->conf[ 'api.' ][ 'e-payment.' ][ 'provider.' ][ 'paymill.' ][ 'keys.' ][ 'private.' ];
		$value = $this->_pObj->local_cObj->cObjGetSingle( $name, $conf );

		$this->_dieApiKeyIsEmpty( $value, 'private' );

		define( 'PAYMILL_API_KEY', $value );
	}

	/**
	 * @return			void
	 * @access			private
	 * @version     8.10.3
	 * @since       8.10.3
	 */
	private function _DefinePaymillPublicKey()
	{
		// conf respects test and live mode
		$name = $this->_pObj->conf[ 'api.' ][ 'e-payment.' ][ 'provider.' ][ 'paymill.' ][ 'keys.' ][ 'public' ];
		$conf = $this->_pObj->conf[ 'api.' ][ 'e-payment.' ][ 'provider.' ][ 'paymill.' ][ 'keys.' ][ 'public.' ];
		$value = $this->_pObj->local_cObj->cObjGetSingle( $name, $conf );

		$this->_dieApiKeyIsEmpty( $value, 'public' );

		define( 'PAYMILL_PUBLIC_KEY', $value );
	}

	/**
	 * @return		string	$path		Path to paymill autoload.php
	 * @access		private
	 * @version   8.10.3
	 * @since     8.10.3
	 */
	private function _Path()
	{
		// RETURN; Paymill PHP Api is loaded
		if( method_exists( '\Paymill\Request', 'create' ) )
		{
			return;
		}

		$path = $this->_pObj->conf[ 'api.' ][ 'e-payment.' ][ 'provider.' ][ 'paymill.' ][ 'files.' ][ 'php.' ][ 'autoload' ];
		$tsProperty = 'api.e-payment.provider.paymill.files.php.autoload';
		return $this->_getPathAbsolute( $path, $tsProperty );
	}

	/**
	 * @param			string	$value	Paymill API key
	 * @param			string	$mode		private || public 
	 * return			void
	 * @version   8.10.3
	 * @since     8.10.3
	 */
	private function _dieApiKeyIsEmpty( $value, $type )
	{
		if( !empty( $value ) )
		{
			return;
		}

		echo ''
		. '<div style="background:red;border:1em solid red;color:white;font-family:sans-serif;padding:1em;text-align:center;">'
		. '<h1>Error</h1>'
		. '<h2>Paymill API Key is empty</h2>'
		. '<p>Please check: api.e-payment.provider.paymill.keys.' . $type . '</p>'
		. '<p><a href="javascript:history.back()" style="color:white;font-weight:bold;">Go Back!</a></p>'
		. '<p>&nbsp;</p>'
		. '<p>Sorry for the trouble.</p>'
		. '<p>' . __METHOD__ . ' (#' . __LINE__ . ')</p>'
		. '</div>'
		;

		die();
	}

	/**
	 * @param			string	$tsProperty	Path of the Typoscript property
	 * return			void
	 * @version   8.10.3
	 * @since     8.10.3
	 */
	private function _diePathIsEmpty( $tsProperty )
	{
		echo ''
		. '<div style="background:red;border:1em solid red;color:white;font-family:sans-serif;padding:1em;text-align:center;">'
		. '<h1>Error</h1>'
		. '<h2>Path to Paymill Autoloader is empty</h2>'
		. '<p>Please check: ' . $tsProperty . '</p>'
		. '<p><a href="javascript:history.back()" style="color:white;font-weight:bold;">Go Back!</a></p>'
		. '<p>&nbsp;</p>'
		. '<p>Sorry for the trouble.</p>'
		. '<p>' . __METHOD__ . ' (#' . __LINE__ . ')</p>'
		. '</div>'
		;

		die();
	}

	/**
	 * @param			string	$path				Path to the autoloader
	 * @param			string	$tsProperty	Path of the Typoscript property
	 * return			void
	 * @version   8.10.3
	 * @since     8.10.3
	 */
	private function _diePathIsWrong( $path, $tsProperty )
	{
		echo ''
		. '<div style="background:red;border:1em solid red;color:white;font-family:sans-serif;padding:1em;text-align:center;">'
		. '<h1>Error</h1>'
		. '<h2>Path to Paymill Autoloader isn\'t proper</h2>'
		. '<p>Path is: ' . $path . ' :(</p>'
		. '<p>Please check: ' . $tsProperty . '</p>'
		. '<p><a href="javascript:history.back()" style="color:white;font-weight:bold;">Go Back!</a></p>'
		. '<p>&nbsp;</p>'
		. '<p>Sorry for the trouble.</p>'
		. '<p>' . __METHOD__ . ' (#' . __LINE__ . ')</p>'
		. '</div>'
		;

		die();
	}

	/**
	 * return			void
	 * @version   8.10.3
	 * @since     8.10.3
	 */
	private function _dieRequirements()
	{
		if( is_array( $this->_pObj->conf ) )
		{
			return;
		}

		echo ''
		. '<div style="background:red;border:1em solid red;color:white;font-family:sans-serif;padding:1em;text-align:center;">'
		. '<h1>Error</h1>'
		. '<h2>Missing Requirement</h2>'
		. '<p>$_pObj->conf isn\'t an array! :(</p>'
		. '<p><a href="javascript:history.back()" style="color:white;font-weight:bold;">Go Back!</a></p>'
		. '<p>&nbsp;</p>'
		. '<p>Sorry for the trouble.</p>'
		. '<p>' . __METHOD__ . ' (#' . __LINE__ . ')</p>'
		. '</div>'
		;

		die();
	}

	/**
	 * @param			string		$path : relative or absolute path to Javascript or CSS
	 * @return		string		$path : absolute path or false in case of an error
	 * @version   8.10.3
	 * @since     8.10.3
	 */
	private function _getPathAbsolute( $path, $keyPathTs )
	{
		// RETURN path is empty
		if( empty( $path ) )
		{
			$this->_diePathIsEmpty( $keyPathTs );
		}

		// URL or EXT:...
		$arr_parsed_url = parse_url( $path );
		if( isset( $arr_parsed_url[ 'scheme' ] ) )
		{
			if( $arr_parsed_url[ 'scheme' ] == 'EXT' )
			{
				unset( $arr_parsed_url[ 'scheme' ] );
			}
		}

		// link to a file
		$bool_file_exists = true;
		if( !isset( $arr_parsed_url[ 'scheme' ] ) )
		{
			$onlyRelative = 1;
			$relToTYPO3_mainDir = 0;
			$absPath = \TYPO3\CMS\Core\Utility\GeneralUtility::getFileAbsFileName( $path, $onlyRelative, $relToTYPO3_mainDir );
			if( !file_exists( $absPath ) )
			{
				$bool_file_exists = false;
				$this->_diePathIsWrong( $path, $keyPathTs );
			}
			// relative path
			$path = preg_replace( '%' . PATH_site . '%', null, $absPath );
		}

		if( !file_exists( $path ) )
		{
			$this->_diePathIsWrong( $path, $keyPathTs );
		}

		return $path;
	}

}
