<?php

namespace Netzmacher\Caddy\Lib\EPayment\Paymill;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2014-2020 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Paymill e-payment modul for the 'caddy' extension.
 *
 * @author	Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage    tx_caddy
 * @version     8.10.3
 * @since       4.0.5
 */
class Main extends \Netzmacher\Caddy\AbstractPlugin
{

	public $extKey = 'caddy';
	public $prefixId = 'tx_caddy_paymill_lib';
	public $scriptRelPath = 'Classes/Lib/EPayment/Paymill/Main.php';
	public $conf = null; // array    : current TypoScript configuration
	public $drs = null; // object   : instance of drs class.
	public $local_cObj = null;
	private $pObj = null; // object   : parent object
	private $pid = null; // integer  : pid of the current page
	private $prompts = array(); // array    : prompts
	private $clientResponse = null; // object   : Paymill payment response object
	private $paymentResponse = null; // object   : Paymill payment response object

	/**
	 * getBankAccount( )  :
	 *
	 * @return	string
	 * @access public
	 * @version    4.0.6
	 * @since      4.0.6
	 */

	public function getBankAccount()
	{
		return $this->paymentResponse->getAccount();
	}

	/**
	 * getBankCode( )  :
	 *
	 * @return	string
	 * @access public
	 * @version    4.0.6
	 * @since      4.0.6
	 */
	public function getBankCode()
	{
		return $this->paymentResponse->getCode();
	}

	/**
	 * getBankHolder( )  :
	 *
	 * @return	string
	 * @access public
	 * @version    4.0.6
	 * @since      4.0.6
	 */
	public function getBankHolder()
	{
		return $this->paymentResponse->getHolder();
	}

	/**
	 * getCardMonth( )  :
	 *
	 * @return	string
	 * @access public
	 * @version    4.0.6
	 * @since      4.0.6
	 */
	public function getCardMonth()
	{
		return $this->paymentResponse->getExpireMonth();
	}

	/**
	 * getCardHolder( )  :
	 *
	 * @return	string
	 * @access public
	 * @version    4.0.6
	 * @since      4.0.6
	 */
	public function getCardHolder()
	{
		return $this->paymentResponse->getCardHolder();
	}

	/**
	 * getCardNumber( )  :
	 *
	 * @return	string
	 * @access public
	 * @version    4.0.6
	 * @since      4.0.6
	 */
	public function getCardNumber()
	{
		return $this->paymentResponse->getLastFour();
	}

	/**
	 * getCardYear( )  :
	 *
	 * @return	string
	 * @access public
	 * @version    4.0.6
	 * @since      4.0.6
	 */
	public function getCardYear()
	{
		return $this->paymentResponse->getExpireYear();
	}

	/**
	 * getPaymillClientId( )  :
	 *
	 * @return	object
	 * @access public
	 * @version    4.0.6
	 * @since      4.0.6
	 */
	public function getPaymillClientId()
	{
		return $this->clientResponse->getId();
	}

	/**
	 * getPaymillPaymentId( )  :
	 *
	 * @return	object
	 * @access public
	 * @version    4.0.6
	 * @since      4.0.6
	 */
	public function getPaymillPaymentId()
	{
		return $this->paymentResponse->getId();
	}

	/**
	 * getPaymillToken( ) :
	 *
	 * @return	string		$token :
	 * @access public
	 * @version    4.0.5
	 * @since      4.0.5
	 */
	public function getPaymillToken()
	{
		$token = $this->piVars[ 'e-payment' ][ 'paymill' ][ 'token' ];
		if( !$token )
		{
			$token = $this->sessionGetToken();
		}
		return $token;
	}

	/*	 * *********************************************
	 *
	 * Paymill
	 *
	 * ******************************************** */

	/**
	 * paymillError( ):
	 *
	 * @param	string		$prompt  :
	 * @return	void
	 * @access private
	 * @version     4.0.5
	 * @since       4.0.5
	 */
	private function paymillError( $prompt )
	{

		// notice (grey) = secondary, info (blue) = [empty!], ok (green) = success, error (red) = alert

		switch( true )
		{
			case( empty( $prompt ) ):
				$prompt = $this->pi_getLL( 'prompt-without-any-content' )
								. '<br />'
								. $this->pi_getLL( 'error-assumption-new-object-with-wasted-token' )
				;
				$this->prompts[] = 'SERVER_PROMPT_WICLOSE_SUBPART|alert alert-danger|' . $prompt;
				$this->prompts[] = 'SERVER_PROMPT_WICLOSE_SUBPART|secondary alert-info|' . $this->pi_getLL( 'prompt-send-bugreport' );
				break;
			case( $prompt == 'Parameter is mandatory' ):
				$prompt = $this->pi_getLL( 'error-parameter-is-mandatory' );
				$this->prompts[] = 'SERVER_PROMPT_WICLOSE_SUBPART||' . $prompt;
				break;
			default:
				$this->prompts[] = 'SERVER_PROMPT_WICLOSE_SUBPART|alert alert-danger|' . $prompt;
				$this->prompts[] = 'SERVER_PROMPT_WICLOSE_SUBPART|secondary alert-info|' . $this->pi_getLL( 'prompt-send-bugreport' );
				break;
		}

		return $this->prompts;
	}

	/**
	 * paymillCheckAccount( ) : This checks the last payment method with e-payment.
	 * 													Will run, if e-payment method was selected, but curretn method isn't any e-payment method!
	 * 													Maybe, this is an unwanted effect. 180312.
	 *
	 * @return	mixed		$error  : returns an error array in case of an error / exception
	 * @access public
	 * @version     8.8.1
	 * @since       4.0.5
	 */
	public function paymillCheckAccount()
	{
		$prompts = array();

		$account = $this->paymentResponse->getAccount();
		$lastFour = $this->paymentResponse->getLastFour();
		$type = $this->paymentResponse->getType();

		switch( true )
		{
			case(!empty( $account ) ):
			case(!empty( $lastFour ) ):
			// #i0201, 180121, dwildt, 1+
			case($type == 'debit' ):
				return null;
			// no error
			default:
				$error = $this->pi_getLL( 'error-account-empty' );
				$prompts = $this->paymillError( $error );
				break;
		}

		unset( $account );
		unset( $lastFour );

		return $prompts;
	}

	/**
	 * paymillInit( ):
	 *
	 * @return	boolean   true, if paymill is initiated. false, if paymill isn't initiated.
	 * @access public
	 * @version     7.4.5
	 * @since       4.0.5
	 */
	public function paymillInit()
	{
		$this->zz_dieIfPaymillIsntLoaded();
		if( !$this->paymillInitWiTokenOnly() )
		{
			$this->zz_debugToConsole( 'No token. No init needed.', __METHOD__, __LINE__ );
			return false;
		}
		$this->zz_debugToConsole( 'Init with token', __METHOD__, __LINE__ );

		$this->paymillInitVars();
		$this->paymillInitApiKey();
		$this->paymillInitAutoload();
		$prompts = $this->paymillInitPayment();


		if( !empty( $prompts ) )
		{
			// #t1369, 170729, dwildt, 1-/+
			//return true;
			return $prompts;
		}

		return false;
	}

	/**
	 * paymillInitWiTokenOnly( ):
	 *
	 * @return	boolean   true, if there is a paymill token. false, if there isn't any paymill token.
	 * @access public
	 * @version     4.0.5
	 * @since       4.0.5
	 */
	public function paymillInitWiTokenOnly()
	{
		if( $this->getPaymillToken() )
		{
			return true;
		}
		return false;
	}

	/**
	 * paymillInit( ):
	 *
	 * @return	void
	 * @access private
	 * @version     4.0.5
	 * @since       4.0.5
	 */
	private function paymillInitApiKey()
	{
		// conf respects test and live mode
		$name = $this->conf[ 'api.' ][ 'e-payment.' ][ 'provider.' ][ 'paymill.' ][ 'keys.' ][ 'private' ];
		$conf = $this->conf[ 'api.' ][ 'e-payment.' ][ 'provider.' ][ 'paymill.' ][ 'keys.' ][ 'private.' ];
		$value = $this->local_cObj->cObjGetSingle( $name, $conf );

		define( 'PAYMILL_API_KEY', $value );
	}

	/**
	 * paymillInitAutoload( ):
	 *
	 * @return	void
	 * @access private
	 * @version     8.10.3
	 * @since       4.0.5
	 */
	private function paymillInitAutoload()
	{
		// #t2145, 180325, dwildt, -/+
//		// #i0163, 170921, dwildt, 2-
//		//$phpAutoload = $this->conf[ 'api.' ][ 'e-payment.' ][ 'provider.' ][ 'paymill.' ][ 'files.' ][ 'php.' ][ 'autoload' ];
//		//$phpAutoload = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath( 'caddy' ) . $phpAutoload;
//		// #i0163, 170921, dwildt, +
//		$path = $this->conf[ 'api.' ][ 'e-payment.' ][ 'provider.' ][ 'paymill.' ][ 'files.' ][ 'php.' ][ 'autoload' ];
//		$tsProperty = 'api.e-payment.provider.paymill.files.php.autoload';
//		$phpAutoload = $this->zz_getPathAbsolute( $path, $tsProperty );
//		require_once( $phpAutoload );
		$_Autoload = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance( \Netzmacher\Caddy\Lib\EPayment\Paymill\Autoload::class, $this );
		$_Autoload->PaymillPhpAPI();
	}

	/**
	 * paymillInitPayment( ):
	 *
	 * @return	mixed		$error  : returns an error array in case of an error / exception
	 * @access private
	 * @version     4.0.5
	 * @since       4.0.5
	 */
	private function paymillInitPayment()
	{
		$piVarToken = $this->piVars[ 'e-payment' ][ 'paymill' ][ 'token' ];
		$sessionToken = $this->sessionGetToken();

		// token is send twice probably because of a page reload
		if( $piVarToken == $sessionToken )
		{
			return $this->paymillInitPaymentBySession( $sessionToken );
		}

		// token is set by piVar: this is a new token
		if( $piVarToken )
		{
			return $this->paymillInitPaymentByToken();
		}

		return $this->paymillInitPaymentBySession( $sessionToken );
	}

	/**
	 * paymillInitPaymentBySession( ) : Init the paymill payment object by the paymill payment id.
	 *
	 * @return	mixed		$error  : returns an error array in case of an error / exception
	 * @access private
	 * @version     7.5.2
	 * @since       4.0.5
	 */
	private function paymillInitPaymentBySession()
	{
		// #i0229, 180325, dwildt
		$request = new \Paymill\Request( PAYMILL_API_KEY );
		$client = new \Paymill\Models\Request\Client();
		$payment = new \Paymill\Models\Request\Payment();


		$clientId = $this->sessionGetClientId();
		$paymentId = $this->sessionGetPaymentId();

		$data = 'clientId: ' . $clientId . ', paymentId: ' . $paymentId;
		$this->zz_debugToConsole( $data, __METHOD__, __LINE__ );

		try
		{
			$client->setid( $clientId );
			$this->clientResponse = $request->getOne( $client );
			//$this->zz_debugToConsole( $this->clientResponse, __METHOD__, __LINE__ );
			$payment->setid( $paymentId );
			$this->paymentResponse = $request->getOne( $payment );
			//$this->zz_debugToConsole( $this->paymentResponse, __METHOD__, __LINE__ );

			return $this->paymillCheckAccount();
		}
		// 170922, TYPO3 exception method must be disabled!
		catch( \Paymill\Services\PaymillException $e )
		{
			$error = $e->getErrorMessage();
			$error = 'Paymill exception: ' . $error . ' | ' . __METHOD__;
			$prompts = $this->paymillError( $error );
			return $prompts;
		}
	}

	/**
	 * paymillInitPaymentByToken( ) :  Init the paymill payment object by the given token.
	 *                                 BE AWARE: A reinit isn't possible. You will lost all paymill data.
	 *
	 * @return	mixed		$error  : returns an error array in case of an error / exception
	 * @access private
	 * @version     7.5.2
	 * @since       4.0.5
	 */
	private function paymillInitPaymentByToken()
	{
		// #i0229, 180325, dwildt
		$request = new \Paymill\Request( PAYMILL_API_KEY );
		$client = new \Paymill\Models\Request\Client();
		$payment = new \Paymill\Models\Request\Payment();


		try
		{
			$client->setEmail( 'test@test.de' )
							->setDescription( 'This is a temporary user.' );
			$this->clientResponse = $request->create( $client );
			//$this->zz_debugToConsole( $this->clientResponse, __METHOD__, __LINE__ );
			$this->zz_debugToConsole( 'client id: ' . $this->clientResponse->getId(), __METHOD__, __LINE__ );
			$payment->setToken( $this->getPaymillToken() )
							->setClient( $this->clientResponse->getId() );
			$this->paymentResponse = $request->create( $payment );
			//$this->zz_debugToConsole( $this->paymentResponse, __METHOD__, __LINE__ );
			$this->zz_debugToConsole( 'payment id: ' . $this->paymentResponse->getId(), __METHOD__, __LINE__ );
			return $this->paymillCheckAccount();
		}

		// 170922, TYPO3 exception method must be disabled!
		catch( \Paymill\Services\PaymillException $e )
		{
			$error = $e->getErrorMessage();
			$error = 'Paymill exception: ' . $error . ' | ' . __METHOD__;
			$prompts = $this->paymillError( $error );
			return $prompts;
		}
	}

	/**
	 * paymillInitVars( )
	 *
	 * @param	integer		$paymentId  : current payment id. 1: credit card, 2: elv. 3: sepa (elv-iban).
	 * @param	[type]		$pid: ...
	 * @return	void
	 * @access private
	 * @version    4.0.6
	 * @since      4.0.6
	 */
	private function paymillInitVars()
	{
		$this->conf = $this->pObj->conf;
		$this->local_cObj = $GLOBALS[ 'TSFE' ]->cObj;

		$this->pi_setPiVarDefaults();
		$this->pi_loadLL();
		//$this->pi_initPIflexForm( );
	}

	/*	 * *********************************************
	 *
	 * Session
	 *
	 * ******************************************** */

	/**
	 * sessionGetClientId( ):
	 *
	 * @return	string		$value  : paymill client id
	 * @access private
	 * @version     4.0.5
	 * @since       4.0.5
	 */
	private function sessionGetClientId()
	{
		$sesArray = $GLOBALS[ 'TSFE' ]->fe_user->getKey( 'ses', $this->extKey . '_' . $this->pid );
		$value = $sesArray[ 'e-payment' ][ 'paymill' ][ 'client' ][ 'id' ];

		return $value;
	}

	/**
	 * sessionGetPaymentId( ):
	 *
	 * @return	string		$value  : paymill payment id
	 * @access private
	 * @version     4.0.5
	 * @since       4.0.5
	 */
	private function sessionGetPaymentId()
	{
		$sesArray = $GLOBALS[ 'TSFE' ]->fe_user->getKey( 'ses', $this->extKey . '_' . $this->pid );
		$value = $sesArray[ 'e-payment' ][ 'paymill' ][ 'payment' ][ 'id' ];

		return $value;
	}

	/**
	 * sessionGetToken( ):
	 *
	 * @return	string		$value  : paymill token
	 * @access private
	 * @version     4.0.5
	 * @since       4.0.5
	 */
	private function sessionGetToken()
	{
		$sesArray = $GLOBALS[ 'TSFE' ]->fe_user->getKey( 'ses', $this->extKey . '_' . $this->pid );
		$value = $sesArray[ 'e-payment' ][ 'paymill' ][ 'token' ];
//var_dump( __METHOD__, __LINE__, $this->pid, $sesArray['e-payment'] );
		return $value;
	}

	/*	 * *********************************************
	 *
	 * Setting methods
	 *
	 * ******************************************** */

	/**
	 * setParentObject( )  :
	 *
	 * @param	object		$pObj: ...
	 * @return	void
	 * @access public
	 * @version    4.0.6
	 * @since      4.0.6
	 */
	public function setParentObject( $pObj )
	{
		if( !is_object( $pObj ) )
		{
			$prompt = 'ERROR: no parent object!<br />' . PHP_EOL .
							'Sorry for the trouble.<br />' . PHP_EOL .
							'TYPO3 Caddy<br />' . PHP_EOL .
							__METHOD__ . ' (' . __LINE__ . ')';
			die( $prompt );
		}
		$this->pObj = $pObj;

//$prompt = 'debug trail: ' . t3lib_utility_Debug::debugTrail( ) . PHP_EOL .
//          'TYPO3 Caddy<br />' . PHP_EOL .
//        __METHOD__ . ' (' . __LINE__ . ')';
//echo $prompt;

		if( !is_object( $pObj->drs ) )
		{
			$prompt = 'ERROR: no DRS object!<br />' . PHP_EOL .
							'Sorry for the trouble.<br />' . PHP_EOL .
//                'debug trail: ' . t3lib_utility_Debug::debugTrail( ) . PHP_EOL .
							'TYPO3 Caddy<br />' . PHP_EOL .
							__METHOD__ . ' (' . __LINE__ . ')';
			die( $prompt );
		}

		$this->drs = $pObj->drs;
	}

	/**
	 * setPid( )  :
	 *
	 * @param	integer		$pid : ...
	 * @return	void
	 * @access public
	 * @version    4.0.6
	 * @since      4.0.6
	 */
	public function setPid( $pid )
	{
		$this->pid = $pid;
	}

	/**
	 * setPiVars( )  :
	 *
	 * @param	array		$piVars : ...
	 * @return	void
	 * @access public
	 * @version    4.0.6
	 * @since      4.0.6
	 */
	public function setPiVars( $piVars )
	{
		$this->piVars = $piVars;
	}

	/**
	 * zz_debugToConsole( ): 
	 *
	 * @param	mixed		$data: 
	 * @param	string		$method: 
	 * @param	string		$line: 
	 * @return	string		$path : absolute path or false in case of an error
	 * @internal    #i0163
	 * @version     8.9.6
	 * @since       7.5.2
	 */
	private function zz_debugToConsole( $data, $method, $line )
	{
		$promptOutput = $this->conf[ 'debug.' ][ 'console' ];
		\Netzmacher\Caddy\Debug\Console::log( $data, $method, $line, $promptOutput );
		return;
	}

	/**
	 * zz_dieIfPaymillIsntLoaded( ): Returns the absolute path of the given path
	 *
	 * @return	void
	 * @internal    #i0200
	 * @since       8.8.1
	 * @version     8.8.1
	 */
	private function zz_dieIfPaymillIsntLoaded()
	{
		if( \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::isLoaded( 'paymill' ) )
		{
			return;
		}

		$prompt = ''
						. 'ERROR! '
						. 'You like to use paymill, but you haven\'t enabled the extension paymill. Please install and enable it.<br />'
						. '<br />'
						. 'Method: ' . __METHOD__ . ' (#' . __LINE__ . ')<br />'
						. '<br />'
						. 'This is a prompt of the etxension Caddy, e-commerce with TYPO3<br />'
						. 'Sorry for the trouble.<br />'
		;

		die( $prompt );
	}

// #t2145, 180325, dwildt, -
//	/**
//	 * getPathAbsolute( ): Returns the absolute path of the given path
//	 *
//	 * @param	string		$path : relative or absolute path to Javascript or CSS
//	 * @return	string		$path : absolute path or false in case of an error
//	 * @internal    #i0163
//	 * @since       7.5.2
//	 * @version     7.5.2
//	 */
//	private function zz_getPathAbsolute( $path, $keyPathTs )
//	{
//		static $drsHasPrompted = false;
//
//		// RETURN path is empty
//		if( empty( $path ) )
//		{
//			if( !( $this->pObj->drs->drsWarn && !$drsHasPrompted ) )
//			{
//				return false;
//			}
//
//			$prompt = 'file can not be included. Path is empty. Maybe it is ok.';
//			\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[WARN/JSS] ' . $prompt, $this->pObj->extKey, 2 );
//			$prompt = 'Change it? Configure: \'' . $keyPathTs . '\'';
//			\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[HELP/JSS] ' . $prompt, $this->pObj->extKey, 1 );
//			$drsHasPrompted = true;
//			return false;
//		}
//
//		// URL or EXT:...
//		$arr_parsed_url = parse_url( $path );
//		if( isset( $arr_parsed_url[ 'scheme' ] ) )
//		{
//			if( $arr_parsed_url[ 'scheme' ] == 'EXT' )
//			{
//				unset( $arr_parsed_url[ 'scheme' ] );
//			}
//		}
//
//		// link to a file
//		$bool_file_exists = true;
//		if( !isset( $arr_parsed_url[ 'scheme' ] ) )
//		{
//			$onlyRelative = 1;
//			$relToTYPO3_mainDir = 0;
//			$absPath = \TYPO3\CMS\Core\Utility\GeneralUtility::getFileAbsFileName( $path, $onlyRelative, $relToTYPO3_mainDir );
//			if( !file_exists( $absPath ) )
//			{
//				$bool_file_exists = false;
//			}
//			// relative path
//			$path = preg_replace( '%' . PATH_site . '%', null, $absPath );
//		}
//
//		if( $bool_file_exists )
//		{
//			return $path;
//		}
//
//		// No DRS
//		if( !$this->pObj->drs->drsError )
//		{
//			return false;
//		}
//
//		// DRS
//		$prompt = 'Script can not be included. File doesn\'t exist: ' . $path;
//		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[ERROR/JSS] ' . $prompt, $this->pObj->extKey, 3 );
//		$prompt = 'Solve it? Configure: \'' . $keyPathTs . '\'';
//		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[HELP/JSS] ' . $prompt, $this->pObj->extKey, 1 );
//		return false;
//	}

}
