<?php

namespace Netzmacher\Caddy\Lib;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2018-2020 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Plugin for the 'caddy' extension.
 *
 * @author	Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage    tx_caddy
 * @version     8.13.0
 * @since       8.13.0
 */
class Sql
{

	private $_cObj = null; // \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;
	private $_drs = null; // Object : the DRS object of the parent object
	private $_pObj = null; // Object : the parent object

	/**
	 * Row( ) : get product details (title, price from table) from database
	 *
	 * @param			integer	$uid: uid of the current product
	 * @return		array		$row: current product
	 * @access		public
	 * @version		8.13.0
	 * @since			8.13.0
	 */

	public function Row( $uid )
	{
		if( empty( $uid ) )
		{
			return false;
		}

		$this->_init();

		$rowLocalised = $this->_rowLocalised( $uid );
		$rowDefault = $this->_rowDefault( $uid );
		$row = $this->_rowConsolidate( $rowLocalised, $rowDefault, $uid );

		return($row);
	}

	/**
	 * SetParentObject( )  : Returns a caddy with HTML form and HTML options among others
	 *
	 * @param			object		$pObj: parent object
	 * @return		void
	 * @access		public
	 * @version   8.13.0
	 * @since     8.13.0
	 */
	public function SetParentObject( $pObj )
	{
		if( !is_object( $pObj ) )
		{
			$prompt = 'ERROR: no parent object!<br />' . PHP_EOL .
							'Sorry for the trouble.<br />' . PHP_EOL .
							'TYPO3 Caddy<br />' . PHP_EOL .
							__METHOD__ . ' (' . __LINE__ . ')';
			die( $prompt );
		}
		$this->_pObj = $pObj;
	}

	/**
	 * _dieIfError( )  :
	 *
	 * @return		void
	 * @access		private
	 * @version   8.13.0
	 * @since     8.13.0
	 */
	private function _dieIfError( $method, $line, $query )
	{
		$error = $GLOBALS[ 'TYPO3_DB' ]->sql_error();

		if( empty( $error ) )
		{
			return;
		}

		$prompt = '<h1>caddy: SQL-Error</h1>' . PHP_EOL
						. '<p>' . $error . '</p>' . PHP_EOL
						. '<p>' . $query . '</p>' . PHP_EOL
						. '<p>' . PHP_EOL
						. 'Please take care for a proper configuration at plugin.tx_caddy_pi1.db<br />' . PHP_EOL
						. 'Sorry for the trouble.<br />' . PHP_EOL
						. 'TYPO3 Caddy<br />' . PHP_EOL
						. $method . ' (' . $line . ')'
		;
		die( $prompt );
	}

	/**
	 * _init( )  :
	 *
	 * @return		void
	 * @access		private
	 * @version   8.13.0
	 * @since     8.13.0
	 */
	private function _init()
	{
		$this->_initCObj();
		$this->_initDrs();
	}

	/**
	 * _initCObj( )  :
	 *
	 * @return		void
	 * @access		private
	 * @version   8.13.0
	 * @since     8.13.0
	 */
	private function _initCObj()
	{
		$this->_cObj = new \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;
	}

	/**
	 * _initDrs( )  :
	 *
	 * @return		void
	 * @access		private
	 * @version   8.13.0
	 * @since     8.13.0
	 */
	private function _initDrs()
	{
		if( !is_object( $this->_pObj->drs ) )
		{
			$prompt = 'ERROR: no DRS object!<br />' . PHP_EOL .
							'Sorry for the trouble.<br />' . PHP_EOL .
							'TYPO3 Caddy<br />' . PHP_EOL .
							__METHOD__ . ' (' . __LINE__ . ')';
			die( $prompt );
		}
		$this->_drs = $this->_pObj->drs;
	}

	/**
	 * _isLocalised( ) : 
	 *
	 * @return		boolean
	 * @access		private
	 * @version		8.13.0
	 * @since			8.13.0
	 */
	private function _isLocalised( $table )
	{
		switch( true )
		{
			case(!isset( $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'languageField' ] ) ):
			case(!isset( $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'transOrigPointerField' ] ) ):
				return false;
			default:
				return true;
		}
	}

	/**
	 * _rowConsolidate( )  :
	 *
	 * @return		void
	 * @access		private
	 * @version   8.13.0
	 * @since     8.13.0
	 */
	private function _rowConsolidate( $rowLocalised, $rowDefault, $uid )
	{
		$row = $this->_rowConsolidateSysLangMode( $rowLocalised, $rowDefault, $uid );
		if( !empty( $row ) )
		{
			return $row;
		}

		$row = $this->_rowConsolidateL10nMode( $rowLocalised, $rowDefault, $uid );

		return $row;
	}

	/**
	 * _rowConsolidateL10nMode( )  :
	 *
	 * @return		mixed
	 * @access		private
	 * @version   8.13.0
	 * @since     8.13.0
	 */
	private function _rowConsolidateL10nMode( $rowLocalised, $rowDefault, $uid )
	{

		$table = $this->_pObj->conf[ 'db.' ][ 'table' ];

		foreach( array_keys( $rowDefault ) as $caddyKey )
		{
			$field = $this->_pObj->conf[ 'db.' ][ $caddyKey ];
			$l10n_mode = $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ $field ][ 'l10n_mode' ];
			switch( $l10n_mode )
			{
				case('exclude'):
					$row[ $caddyKey ] = $rowDefault[ $caddyKey ];
					break;
				default:
					$row[ $caddyKey ] = $rowLocalised[ $caddyKey ];
					break;
			}
		}
		$row[ 'uid' ] = $uid;
		return $row;
	}

	/**
	 * _rowConsolidateSysLangMode( )  :
	 *
	 * @return		mixed
	 * @access		private
	 * @version   8.13.0
	 * @since     8.13.0
	 */
	private function _rowConsolidateSysLangMode( $rowLocalised, $rowDefault, $uid )
	{

		$sys_language_mode = $GLOBALS[ 'TSFE' ]->sys_language_mode;
		switch( true )
		{
			case(empty( $sys_language_mode )):
				return $rowDefault;
			case($sys_language_mode == 'content_fallback'):
				return false;
			case($sys_language_mode == 'strict'):
				$rowLocalised[ 'uid' ] = $uid;
				return $rowLocalised;
			case($sys_language_mode == 'ignore'):
			default:
				if( empty( $rowLocalised ) )
				{
					return $rowDefault;
				}
				$rowLocalised[ 'uid' ] = $uid;
				return $rowLocalised;
		}
	}

	/**
	 * _rowDefault( )  :
	 *
	 * @return		void
	 * @access		private
	 * @version   8.13.0
	 * @since     8.13.0
	 */
	private function _rowDefault( $uid )
	{
		$table = $this->_pObj->conf[ 'db.' ][ 'table' ];
		$select = $this->_select();
		$where = $this->_whereDefault( $table, $uid );
		$groupBy = null;
		$orderBy = null;
		$limit = 1;

		$res = $GLOBALS[ 'TYPO3_DB' ]->exec_SELECTquery( $select, $table, $where, $groupBy, $orderBy, $limit );
		$query = $GLOBALS[ 'TYPO3_DB' ]->SELECTquery( $select, $table, $where, $groupBy, $orderBy, $limit );
		$this->_dieIfError( __METHOD__, __LINE__, $query );

		$row = $GLOBALS[ 'TYPO3_DB' ]->sql_fetch_assoc( $res );
		return $row;
	}

	/**
	 * _rowLocalised( )  :
	 *
	 * @return		void
	 * @access		private
	 * @version   8.13.0
	 * @since     8.13.0
	 */
	private function _rowLocalised( $uid )
	{
		$table = $this->_pObj->conf[ 'db.' ][ 'table' ];

		if( !$this->_isLocalised( $table ) )
		{
			return false;
		}

		$select = $this->_select();
		$where = $this->_whereLocalised( $table, $uid );
		$groupBy = null;
		$orderBy = null;
		$limit = 1;

		$res = $GLOBALS[ 'TYPO3_DB' ]->exec_SELECTquery( $select, $table, $where, $groupBy, $orderBy, $limit );
		$query = $GLOBALS[ 'TYPO3_DB' ]->SELECTquery( $select, $table, $where, $groupBy, $orderBy, $limit );
		$this->_dieIfError( __METHOD__, __LINE__, $query );

		$row = $GLOBALS[ 'TYPO3_DB' ]->sql_fetch_assoc( $res );
		return $row;
	}

	/**
	 * _select( ) : Get the SQL SELECT statement
	 *
	 * @return		string      $select : SQL SELECT statement
	 * @access		private
	 * @version		8.13.0
	 * @since			8.13.0
	 */
	private function _select()
	{
		$table = $this->_pObj->conf[ 'db.' ][ 'table' ];

		$select = null;

		foreach( ( array ) $this->_pObj->conf[ 'db.' ] as $property => $label )
		{
			switch( true )
			{
				case( $property == 'sql' ):
				case( $property == 'sql.' ):
				case( $property == 'table' ):
				case( empty( $label ) ):
					continue 2;
				default:
					// follow the workflow
					break;
			}
			$fields[] = $this->_selectField( $table, $label, $property );
			unset( $property );
		}
		$fields = array_filter( $fields );
		$select = implode( ', ', $fields );

		return $select;
	}

	/**
	 * _selectField( ) :
	 *
	 * @param			string	$table		: table name
	 * @param			string	$field		: field name
	 * @param			string	$alias		: field alias
	 * @return		string  $element	: Element for the SELECT statement
	 * @access		private
	 * @version		8.13.0
	 * @since			8.13.0
	 */
	private function _selectField( $table, $field, $alias )
	{
		$element = NULL;

		$pos = strpos( $field, '.' );
		if( $pos === FALSE )
		{
			$element = $table . '.' . $field . ' AS ' . $alias;
			return $element;
		}

		// Workaround: $field is a table.field. Don't use it.
		return NULL;
	}

	/**
	 * _whereDefault( ) : Get the SQL WHERE statement
	 *
	 * @return		string      $select : SQL WHERE statement
	 * @access		private
	 * @version		8.13.0
	 * @since			8.13.0
	 */
	private function _whereDefault( $table, $uid )
	{
		$where = $table . '.uid = ' . $uid . ' '
						. $this->_cObj->enableFields( $table )
		;

		return $where;
	}

	/**
	 * _whereLocalised( ) : Get the SQL WHERE statement for getting the localised record
	 *
	 * @return		string      $select : SQL WHERE statement
	 * @access		private
	 * @version		8.13.0
	 * @since			8.13.0
	 */
	private function _whereLocalised( $table, $uid )
	{
		$languageField = $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'languageField' ];
		$transOrigPointerField = $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ][ 'transOrigPointerField' ];

		$where = ' ( ' . $table . '.uid = ' . $uid . ' OR ' . $transOrigPointerField . '  = ' . $uid . ' ) '
						. 'AND ' . $languageField . ' = ' . $GLOBALS[ 'TSFE' ]->sys_language_uid . ' '
						. $this->_cObj->enableFields( $table )
		;

		return $where;
	}

}
