<?php

// If next line is enabled, plugin won't run. 180312, dwildt
//namespace Netzmacher\Caddy\Plugins\pi1;
//use TYPO3\CMS\Frontend\Controller;
use Netzmacher\Caddy\Data\DataPowermailDefaults;
use TYPO3\CMS\Core\Utility2x\DebugUtility;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2013-2020 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  Caddy is a fork of wt_cart (version 1.4.6)
 *  (c) wt_cart 2010-2012 - wt_cart Development Team <info@wt-cart.com>
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * plugin 'Cart to powermail' for the 'caddy' extension.
 *
 * @author	Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package	TYPO3
 * @subpackage	tx_caddy
 * @version     9.1.3
 * @since       1.4.6
 */
class tx_caddy_pi1 extends \Netzmacher\Caddy\AbstractPlugin
{

	public $extKey = 'caddy';
	public $prefixId = 'tx_caddy_pi1';
	public $scriptRelPath = 'Resources/Private/Language/Plugins/pi1/class.tx_caddy_pi1.php';
	public $arr_extConf = null;
	private $accessByIP = null;
	public $caddy = null;
	private $clean = null;
	public $cObj = null;
	public $conf = null;
	public $drs = null;
	public $dynamicMarkers = null;
	public $flexform = null;
	public $gpvar = array();
	public $local_cObj = null;
	private $newProduct = array();
	private $numberDeliveryorderCurrent = null;
	private $numberDeliveryorderRegistry = null;
	private $numberInvoiceCurrent = null;
	private $numberInvoiceRegistry = null;
	private $numberOrderCurrent = null;
	private $numberOrderRegistry = null;
	private $_oDataPowermailDefaults;
	public $pid = null;
	public $powermail = null;
	private $session = null;
	private $template = null;
	public $tmpl = null;

	/*	 * *********************************************
	 *
	 * Main
	 *
	 * ******************************************** */

	/**
	 * the main method of the PlugIn
	 *
	 * @param	string		$content: The PlugIn content
	 * @param	array		$conf: The PlugIn configuration
	 * @return	The		content that is displayed on the website
	 * @version    4.0.5
	 * @since      1.4.6
	 */
	public function main( $content, $conf )
	{
		// page object
		$this->local_cObj = $GLOBALS[ 'TSFE' ]->cObj;

		$this->conf = $conf;
		$this->pi_setPiVarDefaults();
		$this->pi_loadLL();
		// Init extension configuration array
		$this->arr_extConf = unserialize( $GLOBALS[ 'TYPO3_CONF_VARS' ][ 'EXT' ][ 'extConf' ][ $this->extKey ] );

		// #t2145, 180325, dwildt, 3+
		// Forward to Paypal, if current Powermail step is action=create and payment method is paypal (e-payment)
		$_Paypal = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance( \Netzmacher\Caddy\Lib\EPayment\Paymill\Paypal::class, $this );
		$_Paypal->Manager();

		// Init DRS, flexform, gpvars, HTML template, service attributes
		$this->init();

		// Output debugging prompts in debug mode
		$this->debugOutputBeforeRunning();

		// Remove current product
		$this->caddyProductDelete();
		// Update several order values
		$this->caddyUpdate();
		// Add a product
		$this->caddyProductAdd();

		// Get the caddy
		$caddy = $this->caddyRendered();
		$marker = $caddy[ 'marker' ];
		$subparts = $caddy[ 'subparts' ];
		$tmpl = $caddy[ 'tmpl' ];

		// 140109, dwildt, 1+
		$tmpl = $this->htmlActiveSection( $tmpl );
//var_dump( __METHOD__, __LINE__ , $tmpl );
		unset( $caddy );

		$content = $this->powermail->formCss( $content );

		$this->powermailDataToSession();

		$this->send();

		$this->clean();

		$content = $content . $this->MarkerBasedTemplateService->substituteMarkerArrayCached( $tmpl, $marker, $subparts );

		$content = $this->caddyHide( $content );

//$sesArray = $GLOBALS['TSFE']->fe_user->getKey( 'ses', $this->extKey . '_' . $this->pid );
//var_dump( __METHOD__, __LINE__, $sesArray['e-payment'] );
//var_dump( __METHOD__, __LINE__, $this->pObj->local_cObj->data );
		$content = $this->dynamicMarkers->main( $content, $this ); // Fill dynamic locallang or typoscript markers
		#i0149/#t1251, 170729, dwildt, 1+
		$this->powermailDefaultData();

		return $this->pi_wrapInBaseClass( $content );
	}

	/*	 * *********************************************
	 *
	 * Caddy
	 *
	 * ******************************************** */

	/**
	 * caddyHide( )  : Hide the caddy, if client has sent the order
	 *
	 * @param	[type]		$$content: ...
	 * @return	string		$content
	 * @access private
	 * @version    4.0.5
	 * @since      4.0.5
	 */
	private function caddyHide( $content )
	{
		$GP = GeneralUtility::_GET() + GeneralUtility::_POST();

		switch( true )
		{
			case( $GP[ 'tx_powermail_pi1' ][ 'action' ] == 'create' ):
				$content = null;
				break;
			default:
				// Follow the workflow
				break;
		}

		unset( $GP );

		return $content;
	}

	/**
	 * caddyProductAdd( )
	 *
	 * @return	void
	 * @access private
	 * @internal    #t0915
	 * @version     9.1.3
	 * @since      2.0.0
	 */
	private function caddyProductAdd()
	{
		if( !isset( $this->gpvar[ 'uid' ] ) )
		{
			return;
		}

		// add further product to session
		$this->newProduct = $this->session->productGet( $this->gpvar[ 'uid' ] );

		if( $this->newProduct !== false )
		{
			$this->newProduct[ 'qty' ] = $this->gpvar[ 'qty' ];
			$this->newProduct = $this->caddyProductServiceAttributes();
			$this->newProduct = $this->caddyProductVariants();
			// #t4472, 200131, dwildt, 1+
			$this->newProduct = $this->caddyProductTitleWiPackingUnit();
//var_dump(__METHOD__, __LINE__, $this->newProduct);
			$this->session->productAdd( $this->newProduct, $this->pid );
		}
	}

	/**
	 * caddyProductDelete( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function caddyProductDelete()
	{
		// remove product from session
		if( isset( $this->piVars[ 'del' ] ) )
		{
			$this->session->productDelete( $this->pid );
		}
	}

	/**
	 * caddyProductServiceAttributes( )
	 *
	 * @return	void
	 * @access private
	 * @internal #i0077
	 * @version    6.0.9
	 * @since      6.0.9
	 */
	private function caddyProductServiceAttributes()
	{
		$this->caddyProductServiceAttributesSet( 'service_attribute_1' );
		$this->caddyProductServiceAttributesSet( 'service_attribute_2' );
		$this->caddyProductServiceAttributesSet( 'service_attribute_3' );

		return $this->newProduct;
	}

	/**
	 * caddyProductServiceAttributesSet( )
	 *
	 * @return	void
	 * @access private
	 * @version    7.2.0
	 * @since      6.0.9
	 */
	private function caddyProductServiceAttributesSet( $key )
	{
		if( isset( $this->gpvar[ $key ] ) )
		{
			$this->newProduct[ $key ] = $this->gpvar[ $key ];
			return;
		}
	}

	/**
	 * caddyProductTitleWiPackingUnit( ):
	 *
	 * @return	array		item / product
	 * @access private
	 * @internal    #t4472
	 * @version    9.1.3
	 * @since      9.1.3
	 */
	private function caddyProductTitleWiPackingUnit()
	{
		$packingunit = $this->newProduct[ 'packingunit' ];
		$packingunitlabel = $this->newProduct[ 'packingunitlabel' ];
		switch( true )
		{
			case(empty( $packingunit )):
				return $this->newProduct;
			default:
				// follow the workflow
				break;
		}

		$tableField = $this->conf[ 'db.' ][ 'table' ] . '.' . $this->conf[ 'db.' ][ 'packingunitlabel' ];
		if( isset( $this->cObj->data[ $tableField ] ) )
		{
			$prompt = 'FATAL ERROR: unexpected variable @ ' . __METHOD__ . ' (#' . __LINE__ . '). Sorry, but this bug must solved by the developer.';
			die( $prompt );
		}
		$this->cObj->data[ $tableField ] = $packingunitlabel;

		$name = $this->conf[ 'templates.' ][ 'snippets.' ][ 'packingunitlabel' ];
		$conf = $this->conf[ 'templates.' ][ 'snippets.' ][ 'packingunitlabel.' ];
		$packingunitlabelLL = $this->zz_cObjGetSingle( $name, $conf );
		unset( $this->cObj->data[ $tableField ] );

		//var_dump( __METHOD__, __LINE__, $name, $conf, $packingunitlabel, $packingunitlabelLL );

		$title = $this->newProduct[ 'title' ];
		$this->newProduct[ 'title' ] = $title . ' (' . $packingunit . ' ' . $packingunitlabelLL . ')';
		return $this->newProduct;
	}

	/**
	 * caddyProductVariants( ): Sets the fields 
	 *                              * $this->newProduct[ 'grossvariants' ]
	 *                              * $this->newProduct[ 'titlevariants' ]
	 *
	 * @return	void
	 * @access private
	 * @internal    #t0915
	 * @version    7.2.0
	 * @since      7.2.0
	 */
	private function caddyProductVariants()
	{
		static $emptyConf = FALSE;

		if( $emptyConf )
		{
			return $this->newProduct;
		}

		$table = $this->conf[ 'constant_editor.' ][ 'variants.' ][ 'tables.' ][ 'property' ];
		if( empty( $table ) )
		{
			$emptyConf = TRUE;
			$this->zz_drsProductVariantsEmptyTable();
			return $this->newProduct;
		}

		// POST has precedence!
		$GP = GeneralUtility::_POST() + GeneralUtility::_GET();

		$field = 'uid';

		if( !isset( $GP[ $table ][ $field ] ) )
		{
			$this->zz_drsProductVariantsNoGPvar( $table, $field );
			return $this->newProduct;
		}

		$values = $GP[ $table ][ $field ];
		//var_dump( __METHOD__, __LINE__, $values );
		switch( true )
		{
			case( empty( $values ) ):
			case( empty( $values[ 0 ] ) ):
				$this->zz_drsProductVariantsEmptyValues( $table, $field );
				return $this->newProduct;
			default:
				// follow the workflow
				break;
		}

		//var_dump( __METHOD__, __LINE__, $GP, $values );
//    $this->newProduct[ 'variants' ] = array(
//        'properties' => array(
//            $table => $this->caddyProductVariantsItem( $table, $field, $values ),
//        ),
//    );

		$this->caddyProductVariantsItem( $table, $field, $values );

		//var_dump( __METHOD__, __LINE__, $this->newProduct );
		return $this->newProduct;
	}

	/**
	 * caddyProductVariantsItem( ): Sets the fields 
	 *                              * $this->newProduct[ 'grossvariants' ]
	 *                              * $this->newProduct[ 'titlevariants' ]
	 *
	 * @param string $table : property table
	 * @param string $field : here: uid
	 * @param array $values : current values of table.uid of GPvar
	 * @return	void
	 * @access private
	 * @internal    #t0915
	 * @version    7.2.0
	 * @since      7.2.0
	 */
	private function caddyProductVariantsItem( $table, $field, $values )
	{
		$this->zz_drsProductVariantsIsVariants( $table, $field, $values );

		$rows = $this->caddyProductVariantsItemSql( $table, $field, $values );

		$tables = $this->conf[ 'constant_editor.' ][ 'variants.' ][ 'tables.' ];
		$tvTitle = $tables[ 'variant' ] . '.' . $tables[ 'variant.' ][ 'title' ];

		$counter = 0;
		$sumGross = 0.00;

		$titleVariants = array();
		foreach( ( array ) $rows as $row )
		{
			$sumGross = $sumGross + $row[ 'gross' ];
			$titleVariants[] = ''
							. $row[ $tvTitle ] . ': '
							. $row[ 'title' ]
							. '<span class="gross">' . $this->caddyProductVariantsItemPrice( $row[ 'gross' ] ) . '</span>'
			;
			$counter = $counter + 1;
		}

		$csvTitleVariants = implode( ', ', $titleVariants );

		$this->newProduct[ 'titlevariants' ] = $csvTitleVariants;
		$this->newProduct[ 'grossvariants' ] = $sumGross;
		return;
	}

	/**
	 * caddyProductVariantsItemSql( ) : Get the rows out of the databse 
	 *                                  for the given uids in the values
	 *                                  for the current table.field
	 *
	 * @param string $table : property table
	 * @param string $field : here: uid
	 * @param array $values : current values of table.uid of GPvar
	 * @return	void
	 * @access private
	 * @internal    #t0915
	 * @version    7.2.0
	 * @since      7.2.0
	 */
	private function caddyProductVariantsItemSql( $table, $field, $values )
	{
		$tableField = $table . '.' . $field;
		$select = $this->caddyProductVariantsItemSqlSelect();
		$tableFrom = $this->caddyProductVariantsItemSqlTable();
		$where = $tableField . ' IN (' . implode( ',', $values ) . ')';
		$groupBy = null;
		$orderBy = null;
		$limit = null;

		$query = $GLOBALS[ 'TYPO3_DB' ]->SELECTquery( $select, $tableFrom, $where, $groupBy, $orderBy, $limit );
		$rows = $GLOBALS[ 'TYPO3_DB' ]->exec_SELECTgetRows( $select, $tableFrom, $where, $groupBy, $orderBy, $limit );

		$boolPromptGP = false;
		$prompt = NULL;
		$this->zz_drsSql( $query, $prompt, __METHOD__, __LINE__, $boolPromptGP );

		return $rows;
	}

	/**
	 * caddyProductVariantsItemPrice( )
	 *
	 * @param	array		$product :
	 * @return	array		$markerArray
	 * @access private
	 * @internal    #t0915, #i0221
	 * @version    8.9.5
	 * @since       7.2.0
	 */
	private function caddyProductVariantsItemPrice( $value )
	{
		$tables = $this->conf[ 'constant_editor.' ][ 'variants.' ][ 'tables.' ];
		if( empty( $tables[ 'property.' ][ 'gross' ] ) )
		{
			return;
		}

		$valueCcurrency = $this->zz_floatToCurrency( $value );
		$price = ' (+' . $valueCcurrency . ')';

		return $price;
	}

	/**
	 * caddyProductVariantsItemSqlSelect( )
	 *
	 * @return	void
	 * @access private
	 * @internal    #t0915, #i0221
	 * @version    8.9.5
	 * @since      7.2.0
	 */
	private function caddyProductVariantsItemSqlSelect()
	{
		$tables = $this->conf[ 'constant_editor.' ][ 'variants.' ][ 'tables.' ];

		$tpUid = $tables[ 'property' ] . '.uid';
		if( !empty( $tables[ 'property.' ][ 'gross' ] ) )
		{
			$tpGross = $tables[ 'property' ] . '.' . $tables[ 'property.' ][ 'gross' ] . " AS 'gross'";
		}
		$tpTitle = $tables[ 'property' ] . '.' . $tables[ 'property.' ][ 'title' ] . " AS 'title'";

		$tvTitle = $tables[ 'variant' ] . '.' . $tables[ 'variant.' ][ 'title' ];
		$tvUid = $tables[ 'variant' ] . '.uid';

		$selectFields = array(
				$tpUid,
				$tpTitle,
				$tvUid . " AS '" . $tvUid . "'",
				$tvTitle . " AS '" . $tvTitle . "'",
		);

		if( !empty( $tpGross ) )
		{
			// #i0268, 181128, dwildt, 1-/+
			//$selectFields = $selectFields + array( $tpGross );
			$selectFields[] = $tpGross;
		}

		$select = implode( ', ', $selectFields );
		return $select;
	}

	/**
	 * caddyProductVariantsItemSqlTable( )
	 *
	 * @return	void
	 * @access private
	 * @internal    #t0915
	 * @version    7.2.0
	 * @since      7.2.0
	 */
	private function caddyProductVariantsItemSqlTable()
	{
		$tables = $this->conf[ 'constant_editor.' ][ 'variants.' ][ 'tables.' ];
		$relations = $this->conf[ 'constant_editor.' ][ 'variants.' ][ 'relation.' ];

		$tpRelation = $tables[ 'property' ] . '.' . $relations[ 'property2variant' ];
		$tvUid = $tables[ 'variant' ] . '.uid';

		$tableFrom = $tables[ 'property' ]
						. ' LEFT JOIN ' . $tables[ 'variant' ]
						. ' ON ' . $tpRelation . ' = ' . $tvUid
		;

		return $tableFrom;
	}

	/**
	 * caddyRendered( )
	 *
	 * @return	array		$caddy : ...
	 * @access private
	 * @version    3.0.1
	 * @since      2.0.0
	 */
	private function caddyRendered()
	{
		// #54634, 131128, dwildt, 1-
		//$arrReturn = $this->caddy->caddy( );
		// #54634, 131128, dwildt, 1+
		$arrReturn = $this->caddy->caddy( $this->pid );
		return $arrReturn;
	}

	/**
	 * caddyUpdate( )
	 *
	 * @return	void
	 * @access private
	 * @version    6.0.9
	 * @since      2.0.0
	 */
	private function caddyUpdate()
	{
		// #i0080, 150212, dwildt, +
		$this->caddyUpdateItemDelete();
		$this->caddyUpdateItemsUpdate();
		$this->caddyUpdateOptions();
	}

	/**
	 * caddyUpdateItemDelete( ) : Deletes an item, if there is an item set to 0.
	 *
	 * @return	void
	 * @access private
	 * @internal #i0080
	 * @version    6.0.9
	 * @since      6.0.9
	 */
	private function caddyUpdateItemDelete()
	{
		if( !isset( $this->piVars[ 'qty' ] ) )
		{
			return;
		}

		foreach( ( array ) $this->piVars[ 'qty' ] as $uid => $quantity )
		{
			if( $quantity > 0 )
			{
				continue;
			}
			$this->piVars[ 'del' ] = $uid;
			$this->caddyProductDelete();
			return;
		}

		return;
	}

	/**
	 * caddyUpdateItemsUpdate( ) : Updates the caddy, if
	 *
	 * @return	void
	 * @access private
	 * @internal #i0080
	 * @version    7.5.8
	 * @since      6.0.9
	 */
	private function caddyUpdateItemsUpdate()
	{
		//var_dump( __METHOD__, __LINE__, $this->piVars, $this->pid  );

		switch( TRUE )
		{
// #i0175, 171125, dwildt, 2-			
//			case(isset( $this->piVars[ 'del' ] )):
//				return;
			case(!isset( $this->piVars[ 'qty' ] )):
				return;
			case(!is_array( $this->piVars[ 'qty' ] )):
				return;
			default:
				// Follow the workflow
				break;
		}

		//var_dump( __METHOD__, __LINE__, $this->piVars );
		$this->session->quantityUpdate( $this->pid );
	}

	/**
	 * caddyUpdateOptions( )
	 *
	 * @return	void
	 * @access private
	 * @version    4.0.4
	 * @since      2.0.0
	 */
	private function caddyUpdateOptions()
	{
		switch( TRUE )
		{
			case(isset( $this->piVars[ 'del' ] )):
				return;
			case(!intval( $this->piVars[ 'updateByCaddy' ] )):
				// RETURN : Don't update options, if form isn't by itself
				return;
			default:
				// Follow the workflow
				break;
		}

		$this->caddyUpdateOptionsInsurance();
		$this->caddyUpdateOptionsPayment();
		$this->caddyUpdateOptionsRecyclingfee();
		$this->caddyUpdateOptionsShipping();
		$this->caddyUpdateOptionsSpecials();
	}

	/**
	 * caddyUpdateOptionsInsurance( )
	 *
	 * @return	void
	 * @access private
	 * @internal	 #i0286
	 * @version    8.17.0
	 * @since      8.17.0
	 */
	private function caddyUpdateOptionsInsurance()
	{
		$insurance = $this->session->insuranceGet( $this->pid );
		if( isset( $this->piVars[ 'insurance' ] ) )
		{
			$insurance = $this->piVars[ 'insurance' ];
		}
		$this->session->insuranceUpdate( $insurance, $this->pid );
	}

	/**
	 * caddyUpdateOptionsPayment( )
	 *
	 * @return	void
	 * @access private
	 * @version    4.0.6
	 * @since      4.0.6
	 */
	private function caddyUpdateOptionsPayment()
	{
		$payment = $this->session->paymentGet( $this->pid );
		if( isset( $this->piVars[ 'payment' ] ) )
		{
			$payment = $this->piVars[ 'payment' ];
		}

		$this->session->paymentUpdate( $payment, $this->pid );
	}

	/**
	 * caddyUpdateOptionsRecyclingfee( )
	 *
	 * @return	void
	 * @access private
	 * @internal	 #i0286
	 * @version    8.17.0
	 * @since      8.17.0
	 */
	private function caddyUpdateOptionsRecyclingfee()
	{
		$recyclingfee = $this->session->recyclingfeeGet( $this->pid );
		if( isset( $this->piVars[ 'recyclingfee' ] ) )
		{
			$recyclingfee = $this->piVars[ 'recyclingfee' ];
		}
		$this->session->recyclingfeeUpdate( $recyclingfee, $this->pid );
	}

	/**
	 * caddyUpdateOptionsShipping( )
	 *
	 * @return	void
	 * @access private
	 * @version    4.0.6
	 * @since      4.0.6
	 */
	private function caddyUpdateOptionsShipping()
	{
		$shipping = $this->session->shippingGet( $this->pid );
		if( isset( $this->piVars[ 'shipping' ] ) )
		{
			$shipping = $this->piVars[ 'shipping' ];
		}
		$this->session->shippingUpdate( $shipping, $this->pid );
	}

	/**
	 * caddyUpdateOptionsSpecials( )
	 *
	 * @return	void
	 * @access private
	 * @version    4.0.6
	 * @since      4.0.6
	 */
	private function caddyUpdateOptionsSpecials()
	{
		// #54858, 140109, dwildt, 1-
		//$special = null;
		// #54858, 140109, dwildt, 1+
		$special = $this->session->specialGet( $this->pid );
//var_dump( __METHOD__, __LINE__, $special );
		// 140202, dwildt, 1-
		//if( isset( $this->piVars['specials'] ) )
		// 140202, dwildt, 1+
		if( isset( $this->piVars[ 'updateByCaddy' ][ 'specials' ] ) )
		{
			$special = $this->piVars[ 'specials' ];
		}
//var_dump( __METHOD__, __LINE__, $special );
		// #54858, 140109, dwildt, 1-
		//$this->session->specialUpdate( $this->piVars['specials'], $this->pid );
		// #54858, 140109, dwildt, 1+
		$this->session->specialUpdate( $special, $this->pid );
	}

	/*	 * *********************************************
	 *
	 * Clean
	 *
	 * ******************************************** */

	/**
	 * clean( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function clean()
	{
		$this->clean->main();
	}

	/*	 * *********************************************
	 *
	 * Debug
	 *
	 * ******************************************** */

	/**
	 * debugOutputBeforeRunning( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function debugOutputBeforeRunning()
	{
		// RETURN : Don't output debug prompt
		if( !$this->conf[ 'debug.' ][ 'paramsAndTs' ] )
		{
			return;
		}
		// RETURN : Don't output debug prompt
		// output debug prompt
		// 140127, dwildt, -
		// deprectad since TYPO3 4.5
//    GeneralUtility::debug
//    (
//      $this->session->productsGet( $this->pid ), $this->extKey . ': ' . 'Values in session at the beginning'
//    );
//    GeneralUtility::debug($this->gpvar, $this->extKey . ': ' . 'Given params');
//    GeneralUtility::debug($this->conf, $this->extKey . ': ' . 'Typoscript configuration');
//    GeneralUtility::debug($_POST, $this->extKey . ': ' . 'All POST variables');
		// 140127, dwildt, +
		DebugUtility::debug
						(
						$this->session->productsGet( $this->pid ), $this->extKey . ': ' . 'Values in session at the beginning'
		);
		DebugUtility::debug( $this->gpvar, $this->extKey . ': ' . 'Given params' );
		DebugUtility::debug( $this->conf, $this->extKey . ': ' . 'Typoscript configuration' );
		DebugUtility::debug( $_POST, $this->extKey . ': ' . 'All POST variables' );

		// output debug prompt
	}

	/*	 * *********************************************
	 *
	 * HTML
	 *
	 * ******************************************** */

	/**
	 * htmlActiveSection( ) : Replace the ###ACTIVE### marker
	 *
	 * @param	string		$tmpl: current template
	 * @return	string		$tmpl: handled template
	 * @access private
	 * @version    8.12.2
	 * @since      4.0.4
	 */
	private function htmlActiveSection( $tmpl )
	{
		// #i0073/#156, 150201, dwildt, 4-
//    if ( GeneralUtility::_GP( 'tx_powermail_pi1' ) )
//    {
//      return $tmpl;
//    }
		// Set default value;
		// #i0261, 180912, dwildt, 1-/2+
		//$accordion = 1;
		$section_id = $this->conf[ 'constant_editor.' ][ 'accordion.' ][ 'default.' ][ 'section_id' ];
		$accordion = ( int ) $section_id;

		if( isset( $this->piVars[ 'accordion' ] ) )
		{
			$accordion = ( int ) $this->piVars[ 'accordion' ];
		}

		$marker = '###CADDY_SECTION_' . $accordion . '###';

		// #i0093, 151203, dwildt, 3+
		$name = $this->conf[ '_HTMLMARKER.' ][ 'cssAccordionActive' ];
		$conf = $this->conf[ '_HTMLMARKER.' ][ 'cssAccordionActive.' ];
		$value = $this->zz_cObjGetSingle( $name, $conf );

		$tmpl = str_replace( $marker, ' ' . $value . ' ', $tmpl );
		$tmpl = str_replace( '  ' . $value, ' ' . $value, $tmpl );
		$tmpl = str_replace( $value . '  ', $value . ' ', $tmpl );
		$tmpl = str_replace( '" ' . $value, '"' . $value, $tmpl );
		$tmpl = str_replace( $value . ' "', $value . '"', $tmpl );

		return $tmpl;
	}

	/*	 * *********************************************
	 *
	 * Init
	 *
	 * ******************************************** */

	/**
	 * init( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function init()
	{
		// init flexform
		$this->pi_initPIflexForm();

		$this->initInstances();
		$this->drs->init();
		$this->initFlexform();
		$this->initPid();
		$this->initAccessByIp();
		$this->initTemplate();
		$this->initGetPost();
		$this->initPowermail();
		// #53679, 131115, dwildt, 2+
		// #i0041, initJavaScript must be after powermail because of ###UID_POWERMAIL_FORM###
		$this->initJavascript();
		$this->initDatabase();
		$this->initNumbers();

		$this->caddy->setParentObject( $this );
		$this->caddy->setContentRow( $this->cObj->data );

		$this->session->setParentObject( $this );
//    var_dump( __METHOD__, __LINE__ );
//    die( ":(" );
	}

	/**
	 * initAccessByIp( ): Set the global $bool_accessByIP.
	 *
	 * @return	void
	 * @version 2.0.0
	 * @since   2.0.0
	 */
	private function initAccessByIp()
	{
		// No access by default
		$this->accessByIP = false;

		// Get list with allowed IPs
		$csvIP = $this->flexform->sdefCsvallowedip;
		$currentIP = GeneralUtility :: getIndpEnv( 'REMOTE_ADDR' );

		// Current IP is an element in the list
		$pos = strpos( $csvIP, $currentIP );
		if( !( $pos === false ) )
		{
			$this->accessByIP = true;
		}
		// Current IP is an element in the list
		// DRS
		if( !$this->drs->drsInit )
		{
			return;
		}

		switch( $this->accessByIP )
		{
			case( true ):
				$prompt = 'Access: current IP matchs the list of allowed IP. Result will prompt to the frontend.';
				GeneralUtility::devlog( ' [OK/INIT] ' . $prompt, $this->extKey, -1 );
				break;
			case( false ):
			default:
				$prompt = 'No access: current IP doesn\'t match the list of allowed IP. Result won\'t prompt to the frontend.';
				GeneralUtility::devlog( ' [WARN/INIT] ' . $prompt, $this->extKey, 2 );
				break;
		}
		// DRS
	}

	/**
	 * initDatabase( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function initDatabase()
	{
		$this->initDatabaseTable();
	}

	/**
	 * initDatabaseTable( )
	 *
	 * @return	void
	 * @access private
	 * @version    7.2.6
	 * @since      2.0.0
	 */
	private function initDatabaseTable()
	{
		$table = $this->conf[ 'db.' ][ 'table' ];

		//echo $GLOBALS['TCA'][$table]['columns'] ) )
		// RETURN : TCA is loaded
		// Load the TCA
		// #t0687, 160702, dwildt, 1-
		//GeneralUtility::loadTCA( $table );

		if( !empty( $GLOBALS[ 'TCA' ][ $table ] ) )
		{
			// DRS
			if( $this->drs->drsInit )
			{
				$prompt = $table . ' is an element of the $TCA.';
				GeneralUtility::devlog( ' [INFO/INIT] ' . $prompt, $this->extKey, 0 );
			}
			// DRS
			return;
		}

		// #i0084, 161120, dwildt, ~
		$prompt = '
      <div style="border:1em solid red;color:red;padding:1em;text-align:center">
        <h1>
          ERROR: table isn\'t proper
        </h1>
        <p>
          The name of the given table isn\'t proper "' . $table . '".
        </p>
        <h2>
          What remains to be done?
        </h2>
        <ul>
          <li>
            Please check the configured caddy page ids first.<br />
            See: constant editor > category: [CADDY - PAGES]
          </li>
          <li>
            Please take care of a proper configuration of the typoscript property db.table.<br />
            See: constant editor > category: [CADDY - DATABASE] > table name
          </li>
        </ul>
        <p>
          Caddy - the Shopping Cart
        </p>
        <p>
          Method: ' . __METHOD__ . ' (#' . __LINE__ . ')
        </p>
      </div>
      ';
		die( $prompt );
	}

	/**
	 * initFlexform( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function initFlexform()
	{
		$this->flexform->main();
	}

	/**
	 * initNumbers( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function initNumbers()
	{
		$registry = new \TYPO3\CMS\Core\Registry;
		// #54634, 131229, dwildt, 1-
		//$prefix = 'page_' . $GLOBALS["TSFE"]->id . '_';
		// #54634, 131229, dwildt, 1+
		$prefix = 'page_' . $this->pid . '_';

		$this->numberDeliveryorderRegistry = ( int ) $registry->get( 'tx_caddy', $prefix . 'deliveryorder' );
		$this->numberInvoiceRegistry = ( int ) $registry->get( 'tx_caddy', $prefix . 'invoice' );
		$this->numberOrderRegistry = ( int ) $registry->get( 'tx_caddy', $prefix . 'order' );

		$this->numberDeliveryorderCurrent = $this->numberDeliveryorderRegistry + $this->flexform->originDeliveryorder + 1;
		$this->numberInvoiceCurrent = $this->numberInvoiceRegistry + $this->flexform->originInvoice + 1;
		$this->numberOrderCurrent = $this->numberOrderRegistry + $this->flexform->originOrder + 1;

		// DRS
		if( $this->drs->drsInit )
		{
			$prompt = 'Delivery order number from registry: ' . $this->numberDeliveryorderRegistry;
			GeneralUtility::devlog( ' [INFO/INIT] ' . $prompt, $this->extKey, 0 );
			$prompt = 'Order number from registry: ' . $this->numberOrderRegistry;
			GeneralUtility::devlog( ' [INFO/INIT] ' . $prompt, $this->extKey, 0 );
			$prompt = 'Invoice number from registry: ' . $this->numberInvoiceRegistry;
			GeneralUtility::devlog( ' [INFO/INIT] ' . $prompt, $this->extKey, 0 );
			$prompt = 'Current delivery order number: ' . $this->numberDeliveryorderCurrent;
			GeneralUtility::devlog( ' [INFO/INIT] ' . $prompt, $this->extKey, 0 );
			$prompt = 'Current invoice number: ' . $this->numberInvoiceCurrent;
			GeneralUtility::devlog( ' [INFO/INIT] ' . $prompt, $this->extKey, 0 );
			$prompt = 'Current order number: ' . $this->numberOrderCurrent;
			GeneralUtility::devlog( ' [INFO/INIT] ' . $prompt, $this->extKey, 0 );
		}
		// DRS

		$this->initNumbersSession();
	}

	/**
	 * initNumbersSession( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function initNumbersSession()
	{
		$sesArray = array();
		// get already exting session
		// #54634, 131229, dwildt, 1-
		//$sesArray = $GLOBALS['TSFE']->fe_user->getKey('ses', $this->extKey . '_' . $GLOBALS["TSFE"]->id);
		// #54634, 131229, dwildt, 1+
		$sesArray = $GLOBALS[ 'TSFE' ]->fe_user->getKey( 'ses', $this->extKey . '_' . $this->pid );

		$sesArray[ 'numberDeliveryorderCurrent' ] = $this->numberDeliveryorderCurrent;
		$sesArray[ 'numberInvoiceCurrent' ] = $this->numberInvoiceCurrent;
		$sesArray[ 'numberOrderCurrent' ] = $this->numberOrderCurrent;

		// generate session with session array
		// #54634, 131229, dwildt, 1-
		//$GLOBALS['TSFE']->fe_user->setKey('ses', $this->extKey . '_' . $GLOBALS["TSFE"]->id, $sesArray);
		// #54634, 131229, dwildt, 1+
		$GLOBALS[ 'TSFE' ]->fe_user->setKey( 'ses', $this->extKey . '_' . $this->pid, $sesArray );
		// save session
		$GLOBALS[ 'TSFE' ]->storeSessionData();

		// DRS
		if( $this->drs->drsInit )
		{
			$prompt = 'Current numbers are stored in the session.';
			GeneralUtility::devlog( ' [INFO/INIT] ' . $prompt, $this->extKey, 0 );
		}
		// DRS
	}

	/**
	 * initGetPost( )
	 *
	 * @return	void
	 * @access private
	 * @version    4.0.7
	 * @since      2.0.0
	 */
	private function initGetPost()
	{

		foreach( array_keys( ( array ) $this->conf[ 'api.' ][ 'getpost.' ] ) as $getpostKey )
		{
			if( stristr( $getpostKey, '.' ) )
			{
				continue;
			}

			$name = $this->conf[ 'api.' ][ 'getpost.' ][ $getpostKey ];
			$conf = $this->conf[ 'api.' ][ 'getpost.' ][ $getpostKey . '.' ];

			// 140207, dwildt, +
			$value = $this->zz_cObjGetSingle( $name, $conf );
			if( empty( $value ) && $value !== 0 )
			{
				continue;
			}
			$this->gpvar[ $getpostKey ] = $value;

			// 140207, dwildt, 1-
//      $this->gpvar[ $getpostKey ] = $this->zz_cObjGetSingle( $name, $conf );
		}

		$this->initGetPostQty();
//var_dump( __METHOD__, __LINE__, $this->gpvar );
		return;
	}

	/**
	 * initGetPostQty( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.2
	 * @since      2.0.0
	 */
	private function initGetPostQty()
	{
		if( $this->gpvar[ 'qty' ] === 0 )
		{
			$this->gpvar[ 'qty' ] = 1;
		}
	}

	/**
	 * initInstances( )
	 *
	 * @return	void
	 * @access private
	 * @version    7.4.4
	 * @since      2.0.0
	 */
	private function initInstances()
	{
		$path2lib = ExtensionManagementUtility::extPath( 'caddy' ) . 'Classes/Lib/';

		// #i0149/#t1251, 170729, dwildt, 2+
		$this->_oDataPowermailDefaults = GeneralUtility::makeInstance( DataPowermailDefaults::class );
		$this->_oDataPowermailDefaults->setExtKey( $this->extKey );

		// #i0226, 180312, dwildt, ~
		$this->caddy = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance( \Netzmacher\Caddy\Lib\Caddy\Caddy::class );
		// #i0226, 180312, dwildt, ~
		$this->clean = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance( \Netzmacher\Caddy\Plugins\pi1\Clean::class );
		$this->clean->pObj = $this;
		$this->clean->row = $this->cObj->data;

		// #i0226, 180312, dwildt, ~
		$this->dynamicMarkers = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance( \Netzmacher\Caddy\Lib\Dynamicmarkers::class );

		// Class with methods for get flexform values
		// #i0226, 180312, dwildt, ~
		$this->flexform = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance( \Netzmacher\Caddy\Plugins\pi1\Flexform::class );
		$this->flexform->pObj = $this;
		$this->flexform->row = $this->cObj->data;

		// #i0150, 170729, dwildt: drs must placed below flexform!
		// #i0226, 180312, dwildt, ~
		$this->drs = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance( \Netzmacher\Caddy\Utility2x\Debug\Drs::class );
		// #i0061, 141129, dwildt, 2-/3+
		//$this->drs->pObj = $this;
		//$this->drs->row = $this->cObj->data;
		$this->drs->setExtConf( $this->arr_extConf );
		$this->drs->setFlexform( $this->flexform );
		$this->drs->setRow( $this->cObj->data );

		// #i0226, 180312, dwildt, ~
		$this->javascript = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance( \Netzmacher\Caddy\Plugins\pi1\Javascript::class );
		$this->javascript->pObj = $this;
		$this->javascript->row = $this->cObj->data;

		// #i0226, 180312, dwildt, ~
		$this->powermail = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance( \Netzmacher\Caddy\Userfunc\Powermail::class );
		$this->powermail->pObj = $this;

		// #i0226, 180312, dwildt, ~
		$this->session = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance( \Netzmacher\Caddy\Lib\Session::class );

		// #i0226, 180312, dwildt, ~
		$this->template = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance( \Netzmacher\Caddy\Lib\Template::class );
		$this->template->pObj = $this;
	}

	/**
	 * initJavascript( )
	 *
	 * @return	void
	 * @access private
	 * @internal   #53679
	 * @version    4.0.0
	 * @since      4.0.0
	 */
	private function initJavascript()
	{
		$this->javascript->addJssFilesJqueryPluginsCaddy();
	}

	/**
	 * initPid( ) :
	 *
	 * @return	void
	 * @version 2.0.0
	 * @since   2.0.0
	 */
	private function initPid()
	{
		$pid = ( int ) $this->conf[ 'pid' ];

		switch( true )
		{
			case(!empty( $pid ) ):
				$this->pid = $pid;
				// DRS
				if( $this->drs->drsInit )
				{
					$prompt = 'pid is taken from main.pid and is set to ' . $this->pid;
					GeneralUtility::devlog( ' [INFO/INIT] ' . $prompt, $this->extKey, 0 );
				}
				// DRS
				break;
			case( empty( $pid ) ):
			default:
				$this->pid = $GLOBALS[ 'TSFE' ]->id;
				// DRS
				if( $this->drs->drsInit )
				{
					$prompt = 'main.pid is empty. pid is set to the id of the current page: ' . $this->pid;
					GeneralUtility::devlog( ' [INFO/INIT] ' . $prompt, $this->extKey, 0 );
				}
				// DRS
				break;
		}

		$this->initPidClasses();
	}

	/**
	 * initPidClasses( ) :
	 *
	 * @return	void
	 * @version 7.4.4
	 * @since   4.0.3
	 */
	private function initPidClasses()
	{
		$this->clean->initPidCaddy( $this->pid );
		$this->dynamicMarkers->initPidCaddy( $this->pid );
		// #i0149/#t1251, 170729, dwildt, 1+
		$this->_oDataPowermailDefaults->setPid( $this->pid );
	}

	/**
	 * initPowermail( )
	 *
	 * @return	void
	 * @access private
	 * @internal   #45915
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function initPowermail()
	{
		$this->powermail->init( $this->cObj->data );
	}

	/**
	 * initTemplate( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function initTemplate()
	{
		$this->tmpl = $this->template->main();
	}

	/*	 * *********************************************
	 *
	 * Powermail
	 *
	 * ******************************************** */

	/**
	 * powermailDataToSession( )
	 *
	 * @return	void
	 * @access private
	 * @version    7.4.4
	 * @since      4.0.5
	 */
	private function powermailDataToSession()
	{
		if( !$this->powermailData() )
		{
			return;
		}

		$this->powermailDataToSessionCustomerEmail();
		$this->powermailDataToSessionCustomerName();
		$this->powermailDataToSessionDefaults();
	}

	/**
	 * powermailDataToSessionCustomerEmail( )
	 *
	 * @return	void
	 * @access private
	 * @version    7.4.4
	 * @since      4.0.5
	 */
	private function powermailDataToSessionCustomerEmail()
	{
		$pmUid = $this->flexform->emailCustomerEmail;
		$emailCustomer = $this->powermail->getFieldById( $pmUid );

		// 180326, dwildt: Workflow in context with PayPal
		if( empty( $emailCustomer ) )
		{
			return;
		}

		$sesArray = $GLOBALS[ 'TSFE' ]->fe_user->getKey( 'ses', $this->extKey . '_' . $this->pid );
		$sesArray[ 'powermail' ][ 'values' ][ 'customer' ][ 'email' ] = $emailCustomer;
		$GLOBALS[ 'TSFE' ]->fe_user->setKey( 'ses', $this->extKey . '_' . $this->pid, $sesArray );
		$GLOBALS[ 'TSFE' ]->storeSessionData();
	}

	/**
	 * powermailDataToSessionCustomerEmail( )
	 *
	 * @return	void
	 * @access private
	 * @version    8.10.3
	 * @since      8.10.3
	 */
	private function powermailDataToSessionCustomerName()
	{
		$sesArray = $GLOBALS[ 'TSFE' ]->fe_user->getKey( 'ses', $this->extKey . '_' . $this->pid );

		$pmUid = $this->flexform->invoiceFirstname;
		$invoiceFirstname = $this->powermail->getFieldById( $pmUid );
		$sesArray[ 'invoiceFirstname' ] = $invoiceFirstname;

		$pmUid = $this->flexform->invoiceLastname;
		$invoiceLastname = $this->powermail->getFieldById( $pmUid );
		$sesArray[ 'invoiceLastname' ] = $invoiceLastname;

		$GLOBALS[ 'TSFE' ]->fe_user->setKey( 'ses', $this->extKey . '_' . $this->pid, $sesArray );
		$GLOBALS[ 'TSFE' ]->storeSessionData();
	}

	/**
	 * powermailDefaultData( )
	 *
	 * @return		void
	 * @access		private
	 * @internal	#i0149/#t1251
	 * @version		7.4.4
	 * @since			7.4.4
	 */
	private function powermailDefaultData()
	{
		if( $this->powermailData() )
		{
			return;
		}

		$this->_oDataPowermailDefaults->PowermailDefaultValues();
	}

	/**
	 * powermailDataToSessionDefaults( )
	 *
	 * @return		void
	 * @access		private
	 * @internal	#i0149/#t1251
	 * @version		7.4.4
	 * @since     7.4.4
	 */
	private function powermailDataToSessionDefaults()
	{
		$this->_oDataPowermailDefaults->PowermailPostToCaddySession();
	}

	/**
	 * powermailData( ):
	 *
	 * @return	boolean
	 * @access private
	 * @version     7.4.4
	 * @since       4.0.5
	 */
	private function powermailData()
	{
		$GP = GeneralUtility::_GET() + GeneralUtility::_POST();

		switch( true )
		{
			case( $GP[ 'tx_powermail_pi1' ] ):
				return true;
			default:
				unset( $GP );
				return false;
		}
	}

	/*	 * *********************************************
	 *
	 * Send
	 *
	 * ******************************************** */

	/**
	 * send( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function send()
	{
		$this->sendCustomer();
		$this->sendVendor();
	}

	/**
	 * sendCustomer( )
	 *
	 * @return	void
	 * @access private
	 * @version    7.1.0
	 * @since      2.0.0
	 */
	private function sendCustomer()
	{
		$this->sendCustomerDeliveryorder();
		$this->sendCustomerInvoice();
		$this->sendCustomerOrderconfirmation();
		$this->sendCustomerRevocation();
		$this->sendCustomerTerms();
	}

	/**
	 * sendCustomerDeliveryorder( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function sendCustomerDeliveryorder()
	{
		$sesArray = $GLOBALS[ 'TSFE' ]->fe_user->getKey( 'ses', $this->extKey . '_' . $this->pid );
		switch( $this->flexform->emailDeliveryorderMode )
		{
			case( 'customer' ):
			case( 'all' ):
				$sesArray[ 'sendCustomerDeliveryorder' ] = $this->flexform->pathsDeliveryorder;
				break;
			default:
				unset( $sesArray[ 'sendCustomerDeliveryorder' ] );
				break;
		}
		$GLOBALS[ 'TSFE' ]->fe_user->setKey( 'ses', $this->extKey . '_' . $this->pid, $sesArray );
		$GLOBALS[ 'TSFE' ]->storeSessionData();

		if( !$this->drs->drsInit )
		{
			return;
		}
		$prompt = 'emailDeliveryorderMode: ' . $this->flexform->emailDeliveryorderMode;
		GeneralUtility::devlog( '[INFO] ' . $prompt, $this->extKey, 0 );
		$prompt = 'pathsDeliveryorder: ' . $this->flexform->pathsDeliveryorder;
		GeneralUtility::devlog( '[INFO] ' . $prompt, $this->extKey, 0 );
	}

	/**
	 * sendCustomerInvoice( )
	 *
	 * @return	void
	 * @access private
	 * @version    7.1.0
	 * @since      7.1.0
	 * @internal   #i0109
	 */
	private function sendCustomerInvoice()
	{
		$sesArray = $GLOBALS[ 'TSFE' ]->fe_user->getKey( 'ses', $this->extKey . '_' . $this->pid );
		switch( $this->flexform->emailInvoiceMode )
		{
			case( 'customer' ):
			case( 'all' ):
				$sesArray[ 'sendCustomerInvoice' ] = $this->flexform->pathsInvoice;
				break;
			default:
				unset( $sesArray[ 'sendCustomerInvoice' ] );
				break;
		}
		$GLOBALS[ 'TSFE' ]->fe_user->setKey( 'ses', $this->extKey . '_' . $this->pid, $sesArray );
		$GLOBALS[ 'TSFE' ]->storeSessionData();
	}

	/**
	 * sendCustomerOrderconfirmation( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function sendCustomerOrderconfirmation()
	{
		$sesArray = $GLOBALS[ 'TSFE' ]->fe_user->getKey( 'ses', $this->extKey . '_' . $this->pid );
		switch( $this->flexform->emailOrderconfirmationMode )
		{
			case( 'customer' ):
			case( 'all' ):
				$sesArray[ 'sendCustomerOrderconfirmation' ] = $this->flexform->pathsOrderconfirmation;
				break;
			default:
				unset( $sesArray[ 'sendCustomerOrderconfirmation' ] );
				break;
		}
		$GLOBALS[ 'TSFE' ]->fe_user->setKey( 'ses', $this->extKey . '_' . $this->pid, $sesArray );
		$GLOBALS[ 'TSFE' ]->storeSessionData();

		if( !$this->drs->drsInit )
		{
			return;
		}
		$prompt = 'emailOrderconfirmationMode: ' . $this->flexform->emailOrderconfirmationMode;
		GeneralUtility::devlog( '[INFO] ' . $prompt, $this->extKey, 0 );
		$prompt = 'pathsOrderconfirmation: ' . $this->flexform->pathsOrderconfirmation;
		GeneralUtility::devlog( '[INFO] ' . $prompt, $this->extKey, 0 );
	}

	/**
	 * sendCustomerRevocation( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function sendCustomerRevocation()
	{
		$sesArray = $GLOBALS[ 'TSFE' ]->fe_user->getKey( 'ses', $this->extKey . '_' . $this->pid );
		switch( $this->flexform->emailRevocationMode )
		{
			case( 'customer' ):
			case( 'all' ):
				$sesArray[ 'sendCustomerRevocation' ] = $this->flexform->pathsRevocation;
				break;
			default:
				unset( $sesArray[ 'sendCustomerRevocation' ] );
				break;
		}
		$GLOBALS[ 'TSFE' ]->fe_user->setKey( 'ses', $this->extKey . '_' . $this->pid, $sesArray );
		$GLOBALS[ 'TSFE' ]->storeSessionData();
	}

	/**
	 * sendCustomerTerms( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function sendCustomerTerms()
	{
		$sesArray = $GLOBALS[ 'TSFE' ]->fe_user->getKey( 'ses', $this->extKey . '_' . $this->pid );
		switch( $this->flexform->emailTermsMode )
		{
			case( 'customer' ):
			case( 'all' ):
				$sesArray[ 'sendCustomerTerms' ] = $this->flexform->pathsTerms;
				break;
			default:
				unset( $sesArray[ 'sendCustomerTerms' ] );
				break;
		}
		$GLOBALS[ 'TSFE' ]->fe_user->setKey( 'ses', $this->extKey . '_' . $this->pid, $sesArray );
		$GLOBALS[ 'TSFE' ]->storeSessionData();
	}

	/**
	 * sendVendor( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function sendVendor()
	{
		$this->sendVendorDeliveryorder();
		$this->sendVendorInvoice();
		$this->sendVendorOrderconfirmation();
		$this->sendVendorRevocation();
		$this->sendVendorTerms();
	}

	/**
	 * sendVendorDeliveryorder( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function sendVendorDeliveryorder()
	{
		$sesArray = $GLOBALS[ 'TSFE' ]->fe_user->getKey( 'ses', $this->extKey . '_' . $this->pid );
		switch( $this->flexform->emailDeliveryorderMode )
		{
			case( 'vendor' ):
			case( 'all' ):
				$sesArray[ 'sendVendorDeliveryorder' ] = $this->flexform->pathsDeliveryorder;
				break;
			default:
				unset( $sesArray[ 'sendVendorDeliveryorder' ] );
				break;
		}
		$GLOBALS[ 'TSFE' ]->fe_user->setKey( 'ses', $this->extKey . '_' . $this->pid, $sesArray );
		$GLOBALS[ 'TSFE' ]->storeSessionData();

		if( !$this->drs->drsInit )
		{
			return;
		}
		$prompt = 'emailDeliveryorderMode: ' . $this->flexform->emailDeliveryorderMode;
		GeneralUtility::devlog( '[INFO] ' . $prompt, $this->extKey, 0 );
		$prompt = 'pathsDeliveryorder: ' . $this->flexform->pathsDeliveryorder;
		GeneralUtility::devlog( '[INFO] ' . $prompt, $this->extKey, 0 );
	}

	/**
	 * sendVendorInvoice( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function sendVendorInvoice()
	{
		$sesArray = $GLOBALS[ 'TSFE' ]->fe_user->getKey( 'ses', $this->extKey . '_' . $this->pid );
		switch( $this->flexform->emailInvoiceMode )
		{
			case( 'vendor' ):
			case( 'all' ):
				$sesArray[ 'sendVendorInvoice' ] = $this->flexform->pathsInvoice;
				break;
			default:
				unset( $sesArray[ 'sendVendorInvoice' ] );
				break;
		}
		$GLOBALS[ 'TSFE' ]->fe_user->setKey( 'ses', $this->extKey . '_' . $this->pid, $sesArray );
		$GLOBALS[ 'TSFE' ]->storeSessionData();
	}

	/**
	 * sendVendorOrderconfirmation( )
	 *
	 * @return	void
	 * @access private
	 * @version    7.1.0
	 * @since      7.1.0
	 * @internal   #i0109
	 */
	private function sendVendorOrderconfirmation()
	{
		$sesArray = $GLOBALS[ 'TSFE' ]->fe_user->getKey( 'ses', $this->extKey . '_' . $this->pid );
		switch( $this->flexform->emailOrderconfirmationMode )
		{
			case( 'vendor' ):
			case( 'all' ):
				$sesArray[ 'sendVendorOrderconfirmation' ] = $this->flexform->pathsOrderconfirmation;
				break;
			default:
				unset( $sesArray[ 'sendVendorOrderconfirmation' ] );
				break;
		}
		$GLOBALS[ 'TSFE' ]->fe_user->setKey( 'ses', $this->extKey . '_' . $this->pid, $sesArray );
		$GLOBALS[ 'TSFE' ]->storeSessionData();

		if( !$this->drs->drsInit )
		{
			return;
		}
		$prompt = 'emailOrderconfirmationMode: ' . $this->flexform->emailOrderconfirmationMode;
		GeneralUtility::devlog( '[INFO] ' . $prompt, $this->extKey, 0 );
		$prompt = 'pathsOrderconfirmation: ' . $this->flexform->pathsOrderconfirmation;
		GeneralUtility::devlog( '[INFO] ' . $prompt, $this->extKey, 0 );
	}

	/**
	 * sendVendorRevocation( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function sendVendorRevocation()
	{
		$sesArray = $GLOBALS[ 'TSFE' ]->fe_user->getKey( 'ses', $this->extKey . '_' . $this->pid );
		switch( $this->flexform->emailRevocationMode )
		{
			case( 'vendor' ):
			case( 'all' ):
				$sesArray[ 'sendVendorRevocation' ] = $this->flexform->pathsRevocation;
				break;
			default:
				unset( $sesArray[ 'sendVendorRevocation' ] );
				break;
		}
		$GLOBALS[ 'TSFE' ]->fe_user->setKey( 'ses', $this->extKey . '_' . $this->pid, $sesArray );
		$GLOBALS[ 'TSFE' ]->storeSessionData();
	}

	/**
	 * sendVendorTerms( )
	 *
	 * @return	void
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function sendVendorTerms()
	{
		$sesArray = $GLOBALS[ 'TSFE' ]->fe_user->getKey( 'ses', $this->extKey . '_' . $this->pid );
		switch( $this->flexform->emailTermsMode )
		{
			case( 'vendor' ):
			case( 'all' ):
				$sesArray[ 'sendVendorTerms' ] = $this->flexform->pathsTerms;
				break;
			default:
				unset( $sesArray[ 'sendVendorTerms' ] );
				break;
		}
		$GLOBALS[ 'TSFE' ]->fe_user->setKey( 'ses', $this->extKey . '_' . $this->pid, $sesArray );
		$GLOBALS[ 'TSFE' ]->storeSessionData();
	}

	/*	 * *********************************************
	 *
	 * ZZ
	 *
	 * ******************************************** */

	/**
	 * zz_cObjGetSingle( ) : Renders a typoscript property with cObjGetSingle, if it is an array.
	 *                       Otherwise returns the property unchanged.
	 *
	 * @param	string		$cObj_name  : value or the name of the property like TEXT, COA, IMAGE, ...
	 * @param	array		$cObj_conf  : null or the configuration of the property
	 * @return	string		$value      : unchanged value or rendered typoscript property
	 * @access private
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	private function zz_cObjGetSingle( $cObj_name, $cObj_conf )
	{
		switch( true )
		{
			case( is_array( $cObj_conf ) ):
				$value = $this->cObj->cObjGetSingle( $cObj_name, $cObj_conf );
				break;
			case(!( is_array( $cObj_conf ) ) ):
			default:
				$value = $cObj_name;
				break;
		}

		return $value;
	}

	/**
	 * zz_drsProductVariantsEmptyTable( )
	 *
	 * @return	void
	 * @access private
	 * @internal    #t0915
	 * @version    7.2.0
	 * @since      7.2.0
	 */
	private function zz_drsProductVariantsEmptyTable()
	{
		static $hasPrompt = NULL;

		switch( TRUE )
		{
			case($hasPrompt === TRUE):
			case(!$this->drs->drsVariants):
				return;
			default:
				// follow the workflow: prompts DRS message
				break;
		}

		$hasPrompt = TRUE;

		$prompt = 'constant_editor.variants.tables.property is empty.';
		GeneralUtility::devlog( '[INFO/VARIANTS] ' . $prompt, $this->extKey, 0 );
	}

	/**
	 * zz_drsProductVariantsEmptyValues( )
	 *
	 * @return	void
	 * @access private
	 * @internal    #t0915
	 * @version    7.2.0
	 * @since      7.2.0
	 */
	private function zz_drsProductVariantsEmptyValues( $table, $field )
	{

		if( !$this->drs->drsVariants )
		{
			return;
		}

		$prompt = 'GP[' . $table . '][' . $field . '] is empty.';
		GeneralUtility::devlog( '[INFO/VARIANTS] ' . $prompt, $this->extKey, 0 );
	}

	/**
	 * zz_drsProductVariantsNoGPvar( )
	 *
	 * @return	void
	 * @access private
	 * @internal    #t0915
	 * @version    7.2.0
	 * @since      7.2.0
	 */
	private function zz_drsProductVariantsNoGPvar( $table, $field )
	{
		static $hasPrompt = NULL;

		switch( TRUE )
		{
			case($hasPrompt === TRUE):
			case(!$this->drs->drsVariants):
				return;
			default:
				// follow the workflow: prompts DRS message
				break;
		}

		$hasPrompt = TRUE;

		$prompt = 'GP[ ' . $table . ' ][ ' . $field . '] isn\'t set.';
		GeneralUtility::devlog( '[INFO/VARIANTS] ' . $prompt, $this->extKey, 0 );
	}

	/**
	 * zz_drsProductVariantsIsVariants( )
	 *
	 * @return	void
	 * @access private
	 * @internal    #t0915
	 * @version    7.2.0
	 * @since      7.2.0
	 */
	private function zz_drsProductVariantsIsVariants( $tableField, $csvValues )
	{
		static $hasPrompt = NULL;

		switch( TRUE )
		{
			case($hasPrompt === TRUE):
			case(!$this->drs->drsVariants):
				return;
			default:
				// follow the workflow: prompts DRS message
				break;
		}

		$hasPrompt = TRUE;

		$prompt = 'Variant is ' . $tableField . '=' . $csvValues;
		GeneralUtility::devlog( '[INFO/VARIANTS] ' . $prompt, $this->extKey, 0 );
	}

	/**
	 * zz_drsSql(): 
	 * 
	 * @return	void
	 * @version 7.2.0
	 * @since 7.2.0
	 */
	private function zz_drsSql( $query, $prompt = NULL, $method = __METHOD__, $line = __LINE__, $boolPromptGP = false )
	{
		$error = $GLOBALS[ 'TYPO3_DB' ]->sql_error();
		if( !$error )
		{
			if( !$this->drs->drsSql )
			{
				return;
			}
			$prompt = $query;
			GeneralUtility::devlog( '[OK/SQL] ' . $prompt, $this->extKey, -1 );
			return;
		}

		if( $this->drs->drsError )
		{
			$prompt = $query;
			GeneralUtility::devlog( '[ERROR/SQL] ' . $prompt, $this->extKey, 3 );
			$prompt = $error;
			GeneralUtility::devlog( '[ERROR/SQL] ' . $prompt, $this->extKey, 2 );
		}

		if( $boolPromptGP )
		{
			$GP = GeneralUtility::_GET() + GeneralUtility::_POST();
			$GPprompt = ''
							. '<pre>' . PHP_EOL
							. var_export( $GP, true ) . PHP_EOL
							. '</pre>' . PHP_EOL
			;
		}

		// DRS
		if( $prompt )
		{
			$prompt = '<h2>' . $prompt . '</h2>' . PHP_EOL;
		}
		$prompt = '<h1>caddy: SQL-Error</h1>' . PHP_EOL
						. '<p>' . $error . '</p>' . PHP_EOL
						. '<p>' . $query . '</p>' . PHP_EOL
						. '<h2>' . $query . '</h2>' . PHP_EOL
						. $GPprompt
						. '<p>' . PHP_EOL
						. 'Sorry for the trouble.<br />' . PHP_EOL
						. 'TYPO3 Caddy<br />' . PHP_EOL
						. $method . ' (' . $line . ')'
						. '</p>' . PHP_EOL
		;
		die( $prompt );
	}

	/**
	 * zz_drsDebugLanguageDie( )
	 *
	 * @return	void
	 * @access private
	 * @internal    #t0915
	 * @version    7.2.0
	 * @since      7.2.0
	 */
	private function zz_drsDebugLanguageDie( $method, $line )
	{
		return;

		//var_dump(__METHOD__, __LINE__, $this->conf[ 'api.' ][ 'symbols.' ][ 'langDebug' ]);
		if( !$this->conf[ 'api.' ][ 'symbols.' ][ 'langDebug' ] )
		{
			return;
		}

		$lang = $this->conf[ 'api.' ][ 'symbols.' ][ 'lang' ];
		//var_dump(__METHOD__, __LINE__, $lang, $GLOBALS[ 'TSFE' ]->lang);
		if( $lang == $GLOBALS[ 'TSFE' ]->lang )
		{
			return;
		}

		echo '<div style="border:2em solid red;padding:2em;color:red;">
          <h1 style="color:red;">
            caddy Error
          </h1>
          <h2 style="color:red;">
            Localisation seem\'s to be confused.
          </h2>
          <p>
            Language of the Constant Editor > [CADDY - MAIN NUMBERFORMAT] > Language is "' . $lang . '".<br />
            But frontend language is "' . $GLOBALS[ 'TSFE' ]->lang . '"
          </p>
          <h3 style="color:red;">
            Solution
          </h3>
          <ul>
            <li>
              Recommended: Set Constant Editor > [CADDY - MAIN NUMBERFORMAT] > Language to "' . $GLOBALS[ 'TSFE' ]->lang . '"
            <li>
              Or configure the language properties in the config array of your TypoScript setup.
            </li>
            </li>
          </ul>
          <h3 style="color:red;">
            Background
          </h3>
          <p>
            Some localisation properties like the decimal point or the thousands seperator depends of the values in a locallang file.<br />
            If localisation configuration isn\'t proper, probably you will get wrong decimal points and thousand seperators :( 
          </p>
          <h3 style="color:red;">
            Get rid of this prompt?
          </h3>
          <p>
            You don\'t need to solve the problem:<br />
            If you want to get rid of this prompt, please disable Constant Editor > [CADDY - MAIN NUMBERFORMAT] > Debug prompt
          </p>
          <h3 style="color:red;">
            System
          </h3>
          <p>
            Method: ' . $method . ' at line #' . $line . '
          </p>
          <p>
            Sorry for the trouble.<br />
            TYPO3 Caddy
          </p>
        </div>';
		exit;
	}

	/**
	 * zz_floatToCurrency() : Format the current value as a currncy number format
	 *                        Format depends on TypoScript and locallang.xlf
	 *
	 * @param	float		$value: current price
	 * @return	string
	 * @access private
	 * @version 7.2.0
	 * @since 7.2.0
	 */
	private function zz_floatToCurrency( $value )
	{
		$this->zz_drsDebugLanguageDie( __METHOD__, __LINE__ );

		$dec_point = $this->conf[ 'api.' ][ 'symbols.' ][ 'dec_point' ];
		$decimal = $this->conf[ 'api.' ][ 'symbols.' ][ 'decimal' ];
		$thousands_sep = $this->conf[ 'api.' ][ 'symbols.' ][ 'thousands_sep' ];

		if( empty( $this->conf ) )
		{
			$this->conf = $GLOBALS[ 'TSFE' ]->tmpl->setup[ 'plugin.' ][ 'tx_caddy_pi1.' ];
		}

		$currencySymbol = $this->conf[ 'api.' ][ 'symbols.' ][ 'currencySymbol' ];
		$price = number_format( $value, $decimal, $dec_point, $thousands_sep );
		// print currency symbol before or after price
		if( $this->conf[ 'api.' ][ 'symbols.' ][ 'currencySymbolBeforePrice' ] )
		{
			$price = $currencySymbol . ' ' . $price;
		}
		else
		{
			$price = $price . ' ' . $currencySymbol;
		}

		return $price;
	}

}

if( defined( 'TYPO3_MODE' ) && $TYPO3_CONF_VARS[ TYPO3_MODE ][ 'XCLASS' ][ 'ext/caddy/pi1/class.tx_caddy_pi1.php' ] )
{
	include_once($TYPO3_CONF_VARS[ TYPO3_MODE ][ 'XCLASS' ][ 'ext/caddy/pi1/class.tx_caddy_pi1.php' ]);
}