<?php

namespace Netzmacher\Caddy\Userfunc;

if( !defined( 'TYPO3_MODE' ) )
{
	die( 'Access denied.' );
}

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2020 -  Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 *
 * @author  Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package TYPO3
 * @subpackage caddy
 * @version     9.1.4
 * @since       9.1.4
 */
class ClearSession
{

	static private $extKey = 'caddy';

	/**
	 * sessionDelete( )
	 *
	 * @param	string		: $content  : current content (will be empty)
	 * @param	array			: $conf     : current TypoScript configuration
	 * @param	integer		: $pid      : uid of the page, which contains the caddy plugin
	 * @return	void
	 * @access public
	 * @version    9.1.4
	 * @since      9.1.4
	 */
	static public function SessionDelete( $content = '', $conf = array(), $pid = null )
	{
		$pid = \Netzmacher\Caddy\Userfunc\GetPid::Caddy( $conf );

		unset( $content );
		$drs = false;
		if( $conf[ 'userFunc.' ][ 'drs' ] )
		{
			$drs = true;
			$prompt = 'DRS is enabled by userFunc.drs';
			\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/USERFUNC] ' . $prompt, self::$extKey, 0 );
		}
		if( $drs )
		{
			$prompt = 'Session is cleared.';
			\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/SESSION] ' . $prompt, self::$extKey, 0 );
		}

		self::_increaseNumbersInTheRegistry(  $conf, $pid, $drs );

		$GLOBALS[ 'TSFE' ]->fe_user->setKey( 'ses', self::$extKey . '_' . $pid, array() );
		$GLOBALS[ 'TSFE' ]->storeSessionData();
	}


	/**
	 * sessionDeleteIncreaseNumbers( )
	 *
	 * @param	boolean	$$drs: Should prompted to the DRS?
	 * @param	intger	$pid: pid of the Caddy
	 * @return	void
	 * @access private
	 * @version    9.1.4
	 * @since      9.1.4
	 */
	static private function _increaseNumbersInTheRegistry( $conf, $pid, $drs )
	{
		$products = self::_getProductsFromSession( $pid );

		// RETURN : any product, don't increase numbers!
		if( empty( $products ) )
		{
			if( $drs )
			{
				$prompt = 'Session is empty! Maybe powermail form is sent twice!';
				\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[ERROR/SESSION] ' . $prompt, self::$extKey, 3 );
			}
			return;
		}

		$registry = new \TYPO3\CMS\Core\Registry;
		$prefix = 'page_' . $pid . '_';

		// Get current numbers
		$numberDeliveryorder = ( int ) $registry->get( 'tx_caddy', $prefix . 'deliveryorder' );
		$numberInvoice = ( int ) $registry->get( 'tx_caddy', $prefix . 'invoice' );
		$numberOrder = ( int ) $registry->get( 'tx_caddy', $prefix . 'order' );

		// Increase current numbers
		$numberDeliveryorder = $numberDeliveryorder + 1;
		$numberInvoice = $numberInvoice + 1;
		$numberOrder = $numberOrder + 1;

		// Set current numbers
		$registry->set( 'tx_caddy', $prefix . 'deliveryorder', $numberDeliveryorder );
		$registry->set( 'tx_caddy', $prefix . 'invoice', $numberInvoice );
		$registry->set( 'tx_caddy', $prefix . 'order', $numberOrder );

		if( !$drs )
		{
			return;
		}

		$prompt = 'New delivery order number: ' . $numberDeliveryorder;
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( ' [INFO/SESSION] ' . $prompt, self::$extKey, 0 );
		$prompt = 'New invoice number: ' . $numberInvoice;
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( ' [INFO/SESSION] ' . $prompt, self::$extKey, 0 );
		$prompt = 'New order number: ' . $numberOrder;
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( ' [INFO/SESSION] ' . $prompt, self::$extKey, 0 );
	}

	/**
	 * productsGet( ) : Read products from session
	 *
	 * @param	integer		uid of the page, which contains the caddy plugin
	 * @return	array		array with all products from session
	 * @access private
	 * @version    9.1.4
	 * @since      9.1.4
	 */
	static private function _getProductsFromSession( $pid )
	{
		$sesArray = $GLOBALS[ 'TSFE' ]->fe_user->getKey( 'ses', self::$extKey . '_' . $pid );

		return $sesArray[ 'products' ];
	}
}
