<?php

namespace Netzmacher\Caddy\Utility2x\Debug;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2013-2020 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */



/**
 * plugin 'Cart to powermail' for the 'caddy' extension.
 *
 * @author	Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package	TYPO3
 * @subpackage	tx_caddy
 * @version	7.2.12
 * @since       1.4.6
 */
class Drs
{

	// path to this script relative to the extension dir.
	private $_extKey = 'caddy';
	// Current row
	private $_row = null;
	private $_extConf;
	private $_flexform;
	public $drsError = false;
	public $drsWarn = false;
	public $drsInfo = false;
	public $drsOk = false;
	public $drsCalc = false;
	public $drsClean = false;
	public $drsCobj = false;
	public $drsEpayment = false;
	public $drsFlexform = false;
	public $drsFormula = false;
	public $drsInit = false;
	public $drsJavascript = false;
	public $drsMarker = false;
	public $drsMinMax = false;
	public $drsOptions = false;
	public $drsPowermail = false;
	public $drsQuantity = false;
	public $drsSession = false;
	public $drsSql = false;
	public $drsStockmanager = false;
	public $drsTodo = false;
	public $drsVariants = false;


	/**
	 * _setTrueAll( ): Init the DRS - Development Reportinmg System
	 *
	 * @return	void
	 * @access private
	 * @version    7.2.12
	 * @since      7.2.12
	 */
	private function _setTrueAll()
	{
		$this->_setTrueErrorWarnInfoOk();
		$this->drsCalc = true;
		$this->drsClean = true;
		$this->drsCobj = true;
		$this->drsEpayment = true;
		$this->drsFlexform = true;
		$this->drsFormula = true;
		$this->drsInit = true;
		$this->drsJavascript = true;
		$this->drsMarker = true;
		$this->drsOptions = true;
		$this->drsPowermail = true;
		$this->drsQuantity = true;
		$this->drsSession = true;
		$this->drsSql = true;
		$this->drsStockmanager = true;
		$this->drsTodo = true;
		$this->drsVariants = true;
	}

	/**
	 * _setTrueErrorWarnInfoOk( ): 
	 *
	 * @return	void
	 * @access private
	 * @version    7.2.12
	 * @since      7.2.12
	 */
	private function _setTrueErrorWarnInfoOk()
	{
		$this->drsError = true;
		$this->drsWarn = true;
		$this->drsInfo = true;
		$this->drsOk = true;
		$this->drsTodo = true;
	}

	/**
	 * _zzDrsPromptInit( ): 
	 *
	 * @return	void
	 * @access private
	 * @version    7.2.12
	 * @since      2.0.0
	 */
	private function _zzDrsPromptInit()
	{
		$prompt = 'The DRS - Development Reporting System is enabled: ' . $this->_extConf[ 'debuggingDrs' ];
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/DRS] ' . $prompt, $this->_extKey, 0 );
		$prompt = 'The DRS is enabled by the extension manager.';
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/DRS] ' . $prompt, $this->_extKey, 0 );
		$str_header = $this->_row[ 'header' ];
		$int_uid = $this->_row[ 'uid' ];
		$int_pid = $this->_row[ 'pid' ];
		$prompt = '"' . $str_header . '" (pid: ' . $int_pid . ', uid: ' . $int_uid . ')';
		\TYPO3\CMS\Core\Utility\GeneralUtility :: devlog( '[INFO/DRS] ' . $prompt, $this->_extKey, 0 );
	}

	/**
	 * init( ): Init the DRS - Development Reportinmg System
	 *
	 * @return	void
	 * @access public
	 * @version    2.0.0
	 * @since      2.0.0
	 */
	public function init()
	{
		$this->_initByExtmngr();

		// RETURN : DRS is enabled by the extension manager
		if( $this->drsOk )
		{
			return;
		}

		$this->_initByFlexform();
	}

	/**
	 * _initByExtmngr( ): Init the DRS - Development Reportinmg System
	 *
	 * @return	void
	 * @access private
	 * @version    7.2.12
	 * @since      2.0.0
	 */
	private function _initByExtmngr()
	{

		switch( $this->_extConf[ 'debuggingDrs' ] )
		{
			case( 'Disabled' ):
			case( null ):
				return;
			case( 'Enabled (for debugging only!)' ):
				$this->_setTrueAll();
				break;
			case( 'Quantity' ):
				$this->_setTrueErrorWarnInfoOk();
				$this->drsQuantity = true;
				break;
			case( 'Stockmanager' ):
				$this->_setTrueErrorWarnInfoOk();
				$this->drsStockmanager = true;
				break;
			case( 'Variants' ):
				$this->_setTrueErrorWarnInfoOk();
				$this->drsVariants = true;
				break;
			default:
				$prompt = 'Error: debuggingDrs is undefined.<br />
          value is ' . $this->_extConf[ 'debuggingDrs' ] . '<br />
          <br />
          ' . __METHOD__ . ' line(' . __LINE__ . ')';
				die( $prompt );
		}

		$this->_zzDrsPromptInit();
	}

	/**
	 * _initByFlexform( ): Init the DRS - Development Reportinmg System
	 *
	 * @return	void
	 * @access private
	 * @version    4.0.5
	 * @since      2.0.0
	 */
	private function _initByFlexform()
	{

		// RETURN : parent object doesn't have any flexform
		if( !is_object( $this->_flexform ) )
		{
			return;
		}
		// sdefDrs
		$sheet = 'sDEF';
		$field = 'sdefDrs';
		$this->_flexform->sdefDrs = $this->_flexform->zzFfValue( $sheet, $field, false );
		// sdefDrs
		// Enable the DRS by TypoScript
		if( empty( $this->_flexform->sdefDrs ) )
		{
			return;
		}

		$this->_setTrueAll();

		$prompt = 'The DRS - Development Reporting System is enabled by the flexform (frontend mode).';
		\TYPO3\CMS\Core\Utility\GeneralUtility::devlog( '[INFO/DRS] ' . $prompt, $this->_extKey, 0 );
		$str_header = $this->_row[ 'header' ];
		$int_uid = $this->_row[ 'uid' ];
		$int_pid = $this->_row[ 'pid' ];
		$prompt = '"' . $str_header . '" (pid: ' . $int_pid . ', uid: ' . $int_uid . ')';
		\TYPO3\CMS\Core\Utility\GeneralUtility :: devlog( '[INFO/DRS] ' . $prompt, $this->_extKey, 0 );
	}

	/**
	 * setExtConf( ):
	 *
	 * @param array $extConf: Extension Configuration array
	 * @return	void
	 * @access public
	 * @version    6.0.3
	 * @since      6.0.3
	 */
	public function setExtConf( $extConf )
	{
		$this->_extConf = $extConf;
	}

	/**
	 * setFlexform( ):
	 *
	 * @param object $flexform
	 * @return	void
	 * @access public
	 * @version    6.0.3
	 * @since      6.0.3
	 */
	public function setFlexform( $flexform )
	{
		$this->_flexform = $flexform;
	}

	/**
	 * setRow( ):
	 *
	 * @param array $row: Current row or cObj->data
	 * @return	void
	 * @access public
	 * @version    6.0.3
	 * @since      6.0.3
	 */
	public function setRow( $row )
	{
		$this->_row = $row;
	}

	/**
	 * setTrue( ):
	 *
	 * @return	void
	 * @access public
	 * @version    7.2.12
	 * @since      7.2.12
	 */
	public function setTrue(  )
	{
		$this->_setTrueAll();
	}

}

if( defined( 'TYPO3_MODE' ) && $TYPO3_CONF_VARS[ TYPO3_MODE ][ 'XCLASS' ][ 'ext/caddy/Classes/Utility2x/Debug/Drs.php' ] )
{
	include_once($TYPO3_CONF_VARS[ TYPO3_MODE ][ 'XCLASS' ][ 'ext/caddy/Classes/Utility2x/Debug/Drs.php' ]);
}
?>
