<?php

namespace Netzmacher\Caddy\Utility2x\TCA;

use Netzmacher\Caddy\Userfunc\TYPO3v;

if( !defined( 'TYPO3_MODE' ) )
{
  die( 'Access denied.' );
}

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2016-2020 -  Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Class provides methods for the extension manager.
 *
 * @author      Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package     TYPO3
 * @subpackage  caddy
 * @version     7.0.2
 * @since       7.0.2
 */
class Icons
{
//  private $pathDefault = 'EXT:caddy/ext_icon';
//  private $pathNotype = 'EXT:quick_shop/Resources/Public/Images/Icons/ExtIcon/notype';
//  private $pathPage = 'EXT:quick_shop/Resources/Public/Images/Icons/ExtIcon/page';
//  private $pathRecord = 'EXT:quick_shop/Resources/Public/Images/Icons/ExtIcon/record';
//  private $pathUrl = 'EXT:quick_shop/Resources/Public/Images/Icons/ExtIcon/url';

  /**
   * Get icon array with classes and paths
   *
   * @return      void
   * @version     7.0.2
   * @since       7.0.2
   */
  private function getClassWiIcons( )
  {
    $extRelPath = \Netzmacher\Refresh\Compatibility\Core\Utility\ExtensionManagementUtility::extRelPath( 'caddy' );
    return array(
        array(
            'class' => 'contains-caddy',
            'key' => 'caddy',
            'label' => 'Caddy',
            'source6xGif' => $extRelPath . 'ext_icon.gif',
            'source6xSvg' => $extRelPath . 'ext_icon.svg',
            'source7x' => 'EXT:caddy/ext_icon.svg',
        ),
        array(
            'class' => 'contains-caddymini',
            'key' => 'caddymini',
            'label' => 'Caddy Mini',
            'source6xGif' => $extRelPath . 'ext_icon.gif',
            'source6xSvg' => $extRelPath . 'ext_icon.svg',
            'source7x' => 'EXT:caddy/ext_icon.svg',
        ),
    );
  }

  /**
   * Register Icons
   *
   * @return      void
   * @version     7.0.2
   * @since       7.0.2
   */
  public function registerIcons( )
  {
    switch( TRUE )
    {
      case(TYPO3v::get() < 7006000):
        $this->registerIcons6x();
        return;
      case(TYPO3v::get() >= 7006000):
      default:
        $this->registerIcons7x( );
        return;
    }
  }

  /**
   * Register Icons
   *
   * @return      void
   * @version     7.0.2
   * @since       7.0.2
   */
  public function registerIcons6x()
  {
    return;
  }

  /**
   * Register Icons
   *
   * @return      void
   * @version     7.0.2
   * @since       7.0.2
   */
  public function registerIcons7x( )
  {
    $iconRegistry = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance( \TYPO3\CMS\Core\Imaging\IconRegistry::class );

    foreach( ( array ) $this->getClassWiIcons( ) as $icon )
    {
      $iconRegistry->registerIcon(
              $icon[ 'class' ]
              , \TYPO3\CMS\Core\Imaging\IconProvider\SvgIconProvider::class
              , [ 'source' => $icon[ 'source7x' ] ]
      );
    }
  }

  /**
   * Register Icons
   *
   * @return      void
   * @version     7.0.2
   * @since       7.0.2
   */
  public function tcaPages( )
  {
    switch( TRUE )
    {
      case(TYPO3v::get() < 7006000):
        $this->tcaPages6x( );
        return;
      case(TYPO3v::get() >= 7006000):
      default:
        $this->tcaPages7x( );
        return;
    }
  }

  /**
   * Register Icons
   *
   * @return      void
   * @version     7.0.2
   * @since       7.0.2
   */
  private function tcaPages6x( )
  {
    global $TCA;
    foreach( ( array ) $this->getClassWiIcons( ) as $icon )
    {
      $TCA[ 'pages' ][ 'columns' ][ 'module' ][ 'config' ][ 'items' ][] = array(
          $icon[ 'label' ]
          , $icon[ 'key' ]
          , $icon[ 'source6xGif' ]
      );
      \TYPO3\CMS\Backend\Sprite\SpriteManager::addTcaTypeIcon(
              'pages'
              , $icon[ 'class' ]
              , $icon[ 'source6xSvg' ]
      );
    }
  }

  /**
   * Register Icons
   *
   * @return      void
   * @version     7.0.2
   * @since       7.0.2
   */
  private function tcaPages7x( )
  {
    global $TCA;

//      $TCA[ 'pages' ][ 'columns' ][ 'module' ][ 'config' ][ 'items' ][] = array(
//          '--- Caddy ---'
//          , ''
//          , ''
//      );
    foreach( ( array ) $this->getClassWiIcons( ) as $icon )
    {
      $TCA[ 'pages' ][ 'columns' ][ 'module' ][ 'config' ][ 'items' ][] = array(
          $icon[ 'label' ]
          , $icon[ 'key' ]
          , $icon[ 'class' ]
      );
      $TCA[ 'pages' ][ 'ctrl' ][ 'typeicon_classes' ][ $icon[ 'class' ] ] = $icon[ 'class' ];
    }
//      $TCA[ 'pages' ][ 'columns' ][ 'module' ][ 'config' ][ 'items' ][] = array(
//          'LLL:EXT:caddy/Resources/Private/Language/locallang.xlf:tx_caddy_item'
//          , 'tx_caddy_item' . '-default'
//          , 'tcarecords-' . 'tx_caddy_item' . '-default'
//      );
  }

}
